// FILE: indriver.c                Copyright (c), Symmetric Research, 1999
//
// Program to install dsp32c kernel mode driver.
//
//

//FIX:
//      also write readme that explains how to install regini by hand
//      change to autostart


#include <windows.h>
#include <stdio.h>
#include "dsp32ckd.h"   // for REG_DRIVER constants

// Defines

#define MAX_STR 256

// Helper function prototype

void ErrorMessage( char *msg, int fatal );


// Global variables shared by main and helper function

SC_HANDLE SCManager, SCService;




void main( int argc, char**argv ) {

        HKEY     Dsp32cKey;
        BOOL     Ret;
        LONG     Code, Code1, Code2;
        ULONG    UserAddress, UserSelect;
        ULONG    PortBaseIo, PortBaseSel;
        DWORD    TagId, Disposition, TypesSupported;
        LPCTSTR  DriverName, DriverDesc, LoadGroup;
        LPCTSTR  ParameterKeyName, EventLogKeyName;
        char     DriverPath[MAX_STR], EventMessageFile[MAX_STR];


        // Banner message

        printf( "\nDSP32C Kernel Mode Driver Installation Program\n");
        


        // Get baseio and basesel addresses from user

        if ( ( argc < 2 ) || ( argc > 3 ) ) {
                printf( "\nUsage: indriver baseio [basesel] \n\n" );
                printf( "       Baseio is the base IO port address" );
                printf( " set on the DSP32C board dip switch.\n" );
                printf( "       Basesel is the CPU selection port set on" );
                printf( " the DSPMUL board 2nd dip switch.\n\n" );
                printf( "       Dip switch defaults are 0x280 for baseio" );
                printf( " and 0x2A0 for basesel.\n" );
                printf( "       Basesel is only valid for DSPMUL boards.\n\n" );
                printf( "A typical command line is:\n\n" );
                printf( "       > indriver 0x280\n" );
                exit( 1 );
                }

        
        sscanf( argv[1], "0x%x", &UserAddress );

        if ( (UserAddress < 0x000) || (UserAddress > 0x3E0) ) {
                printf( "\nBaseIO address must be between 0x000 and 0x3E0\n");
                exit( 1 );
                }

        
        if (argc == 3) {
                
                sscanf( argv[2], "0x%x", &UserSelect );

                if ( (UserSelect < 0x000) || (UserSelect > 0x3E0) ) {
                        printf( "\nBaseSel address must be between 0x000 and 0x3E0\n");
                        exit( 1 );
                        }
                }
        else
                UserSelect = UserAddress;


                
        


        // Open Service Control Manager

        SCManager = NULL;
        SCService = NULL;
        
        SCManager = OpenSCManager(
                        NULL,                  // use local machine
                        NULL,                  // use SERVICES_ACTIVE_DATABASE
                        SC_MANAGER_ALL_ACCESS  // full access permission
                        );

        if (!SCManager)
                ErrorMessage("Could not create service manager", 1);
        



        // Create the device driver service.
        //
        // This involves creating a registry key with several 
        // values and adding the service to the services database.  
        // The registry entries can be made by hand.  But in that
        // case, you must reboot the machine before the service 
        // will be included in the services database.  Once the 
        // service is available, it can be started.  The pertinent 
        // registry entries are as follows (notes in parenthesis are
        // just information and are not included in the registry):
        //
        // HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\Dsp32c
        //   DisplayName  = Dsp32c DSP Device Driver    (DriverDesc)
        //   ErrorControl = 0x00000001                  (SERVICE_ERROR_NORMAL)
        //   Group        = Digital Signal Processing Devices  (LoadGroup)
        //   ImagePath    = \??\C:\sr\dsp32c\nt\dll\dsp32c.sys (\??\C:DriverPath)
        //   Start        = 0x00000002                  (SERVICE_AUTO_START)
        //   Tag          = 0x00000001                  (TagId)
        //   Type         = 0x00000001                  (SERVICE_KERNEL_DRIVER)
        //
        //   Start        = 0x00000003                  (SERVICE_DEMAND_START)

        GetCurrentDirectory( MAX_STR, DriverPath );
        lstrcat( DriverPath,  "\\dsp32c.sys");
        
        DriverName  = REG_DRIVER_NAME;
        DriverDesc  = REG_DRIVER_DESC;
        LoadGroup   = REG_DRIVER_GROUP;
        TagId       = 0x1L;

        
        SCService = CreateService(
                        SCManager,             // Service Control Manager
                        DriverName,            // name of device driver
                        DriverDesc,            // description of driver
                        SERVICE_ALL_ACCESS,    // full access permission
                        SERVICE_KERNEL_DRIVER, // type is a kernel driver
                        SERVICE_AUTO_START,    // start at end of booting
//                        SERVICE_DEMAND_START,    // ie net start dsp32c
                        SERVICE_ERROR_NORMAL,  // give warning, but still boot
                        DriverPath,            // location of .sys file
                        LoadGroup,             // loads in custom A/D group
                        &TagId,                // load order within group
                        NULL,                  // depends on no other services
                        NULL,                  // use default ServiceStartName
                        NULL                   // no password
                       );

        if (!SCService)
                ErrorMessage("Could not create device driver service", 1);




        // Add registry entries for device specific parameters

        ParameterKeyName =
                  "System\\CurrentControlSet\\Services\\Dsp32c\\Parameters";
        
        PortBaseIo       = UserAddress;
        PortBaseSel      = UserSelect;

        Code = RegCreateKeyEx(
                       HKEY_LOCAL_MACHINE,      // put under this key
                       ParameterKeyName,        // at this location
                       0,                       // reserved
                       NULL,                    // object type of key
                       REG_OPTION_NON_VOLATILE, // keep value when reboot
                       KEY_ALL_ACCESS,          // full access permission
                       NULL,                    // key handle not inheritable
                       &Dsp32cKey,              // handle to key
                       &Disposition             // indicates key new or existing
                      );

        if (Code != ERROR_SUCCESS)
                ErrorMessage("Failed to create parameter key", 1);


        Code1 = RegSetValueEx(
                    Dsp32cKey,                      // use this key
                    REG_PARM_BASEIO,                // value name
                    0,                              // reserved
                    REG_DWORD,                      // value data type
                    &(CONST BYTE)PortBaseIo,        // ptr to value contents
                    sizeof(ULONG)                   // contents size
                   );
        
        Code2 = RegSetValueEx(
                    Dsp32cKey,                      // use this key
                    REG_PARM_BASESEL,               // value name
                    0,                              // reserved
                    REG_DWORD,                      // value data type
                    &(CONST BYTE)PortBaseSel,       // ptr to value contents
                    sizeof(ULONG)                   // contents size
                   );
        
        
        if ( (Code1 != ERROR_SUCCESS) ||
             (Code2 != ERROR_SUCCESS) )
                printf("Failed writing parameter key values to registry\n");
        
                       
        Code = RegCloseKey( Dsp32cKey );
        
        if (Code != ERROR_SUCCESS) 
                ErrorMessage("Failed to close parameter key", 1);
        

        

        // Add registry entries for interpreting EventViewer entries
        //
        // Note that these entries are NOT deleted when rmdriver.exe
        // is run.  This is so that System Log entries relating to the
        // Dsp32c driver can still be interpreted even when the driver
        // is no longer running.  Except for making the System Log
        // entries less informative, deleting these registry entries
        // by hand produces no bad effects.
        //

        EventLogKeyName  =
              "System\\CurrentControlSet\\Services\\EventLog\\System\\Dsp32c";
        
        TypesSupported   = 0x7L;
        lstrcpy( EventMessageFile, DriverPath );
        lstrcat( EventMessageFile, ";%SystemRoot%\\System32\\IoLogMsg.dll");
                         
        Code = RegCreateKeyEx(
                       HKEY_LOCAL_MACHINE,      // put under this key
                       EventLogKeyName,         // at this location
                       0,                       // reserved
                       NULL,                    // object type of key
                       REG_OPTION_NON_VOLATILE, // keep value when reboot
                       KEY_ALL_ACCESS,          // full access permission
                       NULL,                    // key handle not inheritable
                       &Dsp32cKey,              // handle to key 
                       &Disposition             // indicates key new or existing
                      );

        if (Code != ERROR_SUCCESS) 
                ErrorMessage("Failed creating Event Log Key", 1);


        Code1 = RegSetValueEx(
                    Dsp32cKey,                      // use this key
                    "EventMessageFile",             // value name
                    0,                              // reserved
                    REG_EXPAND_SZ,                  // value data type
                    (CONST BYTE*)EventMessageFile,  // ptr to value contents
                    strlen(EventMessageFile)        // contents size
                    );
        
                              
        Code2 = RegSetValueEx(
                    Dsp32cKey,                      // use this key
                    "TypesSupported",               // value name
                    0,                              // reserved
                    REG_DWORD,                      // value data type
                    &(CONST BYTE)TypesSupported,    // ptr to value contents
                    sizeof(DWORD)                   // contents size
                   );
        
        if ( (Code1 != ERROR_SUCCESS) ||
             (Code2 != ERROR_SUCCESS) )
                printf("Failed writing event log key values to registry\n");
        

        Code = RegCloseKey( Dsp32cKey );
        if (Code != ERROR_SUCCESS)
                ErrorMessage("Failed closing Event Log Key", 1);




        // Indicate install successful
        
        printf( "\n%s driver has been successfully installed.\n", DriverName );

        
        
        // Start Service

        Ret = StartService( SCService,    // service identifier
                            0,             // number of arguments
                            NULL           // pointer to arguments
                          );

        if (!Ret)
                ErrorMessage("Failed to start DSP32C device driver service", 1);
        else
                printf( "\n%s driver was started using port address 0x%X.\n",
                        DriverName, PortBaseIo );
        

        // Close handles (leaving service loaded and running)

        CloseServiceHandle( SCService );
        CloseServiceHandle( SCManager );


}

void ErrorMessage( char *msg, int fatal ) {

        DWORD  Err;
        LPVOID lpMessageBuffer;
        
        // Display local message, error number, and system message
        
        Err = GetLastError();

        FormatMessage(
                      FORMAT_MESSAGE_ALLOCATE_BUFFER |
                      FORMAT_MESSAGE_FROM_SYSTEM,
                      NULL,
                      Err,
                      MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
                      (LPTSTR) &lpMessageBuffer,
                      0,
                      NULL
                     );

        printf( "\n%s.  Error #%d\n\n%s\n", msg, Err, lpMessageBuffer );

        LocalFree( lpMessageBuffer );


        
        // Give additional explanation for common errors
        
        if (Err == ERROR_SERVICE_EXISTS) { // #1073
                printf( "\tNote: Use rmdriver.exe to clear out\n" );
                printf( "\tthe previous version and try again.\n" );
                }
        
        if (Err == ERROR_NO_SYSTEM_RESOURCES) { // #1450
                printf( "\tThis error most likely indicates that the Dsp32c\n" );
                printf( "\tdevice driver could not start because another\n" );
                printf( "\tdevice was using the requested port addresses.\n" );
                printf( "\t\n" );
                printf( "\tOnce the conflict is resolved, you can start\n" );
                printf( "\tthe Dsp32c device driver by typing:\n" );
                printf( "\tnet start dsp32c\n" );
                printf( "\t\n" );
                printf( "\tPlease examine the System Log in the EventViewer\n" );
                printf( "\tfor more details on the resource conflict.  To\n" );
                printf( "\taccess the EventViewer, type eventvwr at the\n" );
                printf( "\tCommandPrompt or select Programs > Administrative\n" );
                printf( "\tTools > EventViewer from the Start menu.\n" );
                }


        // Clean up and exit on fatal errors
        
        if (fatal) {
                if (SCService)  CloseServiceHandle( SCService );
                if (SCManager)  CloseServiceHandle( SCManager );
                exit( 1 );
                }
        
}
