/* File: dspasm.c               Copyright (c), Symmetric Research, 1992-1999
 *
 * SR DSPASM DSP32C assembler.
 *
 *
 *
 */

#include "dspasm.h"





/* DOS COMMAND LINE USAGE:
 *
 *
 *
 *
 *
 */

char *DOS_usage_msg[] = {

     "USAGE: Command line usage ...                                         ",
     "                                                                      ",
     "  dspasm [/{1,2}] [/l] [/s] [/sa] [/sc] [/h] [/n{+,-}] {file}         ",
     "                                                                      ",
     "       /1             single pass assembly                 (default)  ",
     "       /2             two pass assembly                               ",
     "                                                                      ",
     "       /l             generate .dsl listing file                      ",
     "       /s             generate .dss symbol table                      ",
     "       /sa            generate .dsa symbol table                      ",
     "       /sc            generate .dsc symbol table (C .h file format)   ",
     "       /h             generate .dsh hexified object file              ",
     "                                                                      ",
     "       /n+            turn automatic CAU nops on           (default)  ",
     "       /n-            turn automatic CAU nops off                     ",
     "                                                                      ",
     "       file           input source code file                          ",
     "                                                                      ",
     "               where {} is required, and [] is optional               ",

     NULL,

     };


void DOS_usage( void) {

        int i;


        /* Print the command line usage message. */

        for ( i = 0 ; DOS_usage_msg[i] ; i++ )

                printf( "%s\n", DOS_usage_msg[i] );


        /* Return to DOS with an error. */

        exit( 1);


}
void DOS_error( char *msg) {


        /* Print the error message. */

        printf( "ERROR: %s ...", msg);


        /* Return to DOS with an error. */

        exit( 1);
}








/* MAIN:
 *
 * Open files, call parser, etc ...
 *
 *
 *
 *
 */

char
        *filename       = "";           /* input filename               */

unsigned                                /* command line flags           */

        pflag           = 1,            /* number of passes to make     */
        nflag           = 1,            /* nops on or off               */
        hflag           = 0,            /* ASCII hex output             */
        tflag           = 0,            /* dump input tokens            */
        lflag           = 0,            /* object listing               */
        sflag           = 0,            /* symbol listings              */
        saflag          = 0,
        scflag          = 0;

unsigned

        pass            = 1,            /* current assembler pass       */
        nops            = 1;            /* current CAU latency nops     */


int main( int argc, char **argv) {

        static struct command_line_option

                { char *name; unsigned *flag, value; } *t, table[] = {

                        "/1",   &pflag,         1,
                        "/2",   &pflag,         2,

                        "/n+",  &nflag,         1,
                        "/n-",  &nflag,         0,

                        "/h",   &hflag,         1,
                        "/t",   &tflag,         1,

                        "/l",   &lflag,         1,

                        "/s",   &sflag,         1,
                        "/sa",  &saflag,        1,
                        "/sc",  &scflag,        1,

                        NULL,   NULL,           0,

                        };



        /* Get the command line arguments. */

        if ( argc < 2 )

                DOS_usage();


        while ( --argc ) { argv++;


                if ( **argv == '/' ) { /* search for a command line option */


                        t = table;


                            while ( t->name && strcmp( *argv, t->name) )

                                t++;


                            if ( t->name )

                                    *t->flag = t->value;

                            else
                                    DOS_usage();

                        }

                else if ( !*filename )

                        filename = *argv;

                else
                        DOS_usage();

                }




        /* Allocate buffers and begin reading the input file. */

        if ( !allocate_input_buffer() )

                DOS_error( "insufficient memory for input buffer");


        if ( !allocate_output_buffer() )

                DOS_error( "insufficient memory for output buffer");


        if ( !read_input_file() )

                DOS_error( "unable to read input file");





        /* Process the input file with the requested number of passes. */

        printf( "\n");



        /* pass 1 ... */

                pass = 1;

                  nops = nflag;

                printf( "\t pass 1 ...\n");

                FILE_ID(); program(); FINAL_RECORD();



        /* pass 2 ... */

        if ( pflag == 2 ) {

                pass = 2;

                  nops = nflag; reset_input_file(); reset_output_buffer();

                printf( "\t pass 2 ...\n");

                FILE_ID(); program(); FINAL_RECORD();

                }


        printf( "\n");





        /* Generate the object file in the requested format. */

        if ( hflag )

                write_dsh();

        else
                write_dso();




        /* Generate any additional listing files. */

        if ( lflag )                    /* listing file */

                write_dsl();

        if ( sflag )                    /* symbol tables */

                write_sym( 0);

        if ( saflag )

                write_sym( 1);

        if ( scflag )

                write_sym( 2);




        /* Return to DOS with no errors. */

        exit( 0);


}








/* Partial DSPASM grammar:
 *
 *
 *          program ->      ( org | constant | global | function ) program
 *    
 *    
 *    
 *              org ->      <N>
 *    
 *         constant ->      "constant" name = N ( , name = N ... ) ;
 *    
 *           global ->      "byte" variable
 *                          "word" variable
 *                          "long" variable
 *                          "dspf" variable
 *                          "ieee" variable
 *    
 *         variable ->      name dimension initializer ( , variable ... ) ;
 *    
 *    
 *    
 *    
 *         function ->      name { function_body }
 *    
 *    function_body ->      ( local | statement ) function_body
 *    
 *    
 *    
 *    
 *    
 *            local ->      ( named_register | structure )
 *    
 *   named_register ->      "register" name = rN ( , name = rN ... ) ;
 *    
 *        structure ->      "struct" N { field ( , field ... ) } ;
 *    
 *    
 *    
 *
 *        statement ->      ( conditional | instruction )
 *    
 *      conditional ->      if ... else if
 *                          while
 *                          for
 *                          loop
 *                          do
 *    
 *      instruction ->      nop
 *                          call
 *                          return
 *                          goto
 *                          ireturn
 *                          nops_on
 *                          nops_off
 *                          emit
 *                          iff
 *    
 *                          cau integer instruction
 *                          dau floating point instruction
 *    
 *                          label:
 *
 *
 *  See the AT&T DSP32C information manual for CAU and DAU instruction ...
 *
 *
 *
 */

int program( void) {


        program:

            if ( org() || constant() || global() || function() )

                goto program;




        if ( end_of_file() )

                return( 1);

        else
                syntax( "legal global statement");




        return( 0); /* failure */




}
int org( void) {



        /* Parse for a location origin statement, <N>. */

        if ( !match( '<') )

                return( 0); /* failure */


        if ( !hexexp( &pc) )

                syntax( "<N>, where N = hex expression");


        if ( !resolved )

                error( "all <N> locations must be resolved on pass 1");





        /**** KEEP ALIGNED TO 4 BYTE BOUNDARIES. ****/

        if ( pc&0x3 ) {

                warning( "aligning location to 4 byte boundary");

                        pc = (pc+3)&~3L;

                }




        /* Check for the statement terminator. */

        if ( match( '>') )

                return( 1);

        else

                syntax( "legal <N> location directive");




        return( 0); /* failure */



}
int constant( void) {

        SYMBOL *s; unsigned long value;




        /* Parse for a constant statement, constant name = ... */

        if ( !scan( "constant") )

                return( 0); /* failure */




        /* Enter constant names into the global symbol table. */

  next:

        if ( (s = enter( CONS_SYM, gsymbol)) != NULL ) {


                if ( match( '=') ) {


                        if ( hexexp( &value) ) {


                                s->value.l = value;

                                if ( resolved )

                                        s->flags |= RESOLVED;

                                }

                        else

                                syntax( "constant name = N");

                        }

                else

                        syntax( "constant name =");

                }





        /* Check for the statement terminator. */

        if ( match( ',') )

                goto next;


        else if ( match( ';') )

                return( 1);


        else
                syntax( "legal constant name , or ;");




        return( 0); /* failure */




}



int datasize[] = {                      /* data type sizes in bytes */

        /* type = 0, BYTE */    1,
        /* type = 1, WORD */    2,
        /* type = 2, LONG */    4,
        /* type = 3, DSPF */    4,
        /* type = 4, IEEE */    4,

        };



#define SINGLE          1               /* dflag values in global() */
#define DIMENSIONED     2
#define UNDIMENSIONED   3



int global( void) {

        int type, dflag; SYMBOL *s; unsigned long N;


            static int symtype[] = {  /* map from data to symbol types */

                /* datatype = 0, BYTE */    BYTE_SYM,
                /* datatype = 1, WORD */    WORD_SYM,
                /* datatype = 2, LONG */    LONG_SYM,
                /* datatype = 3, DSPF */    DSPF_SYM,
                /* datatype = 4, IEEE */    IEEE_SYM,

                };






        /* Parse for global data, {byte,word,long,dspf,ieee} var name... */

        if ( !datatype( &type) )

                return( 0); /* failure */




        /* Enter variable names into the global symbol table. */

  next:

        if ( (s = enter( symtype[ type], gsymbol)) != NULL ) {


            s->value.l = pc; s->flags |= RESOLVED;




                /* Get a possible array dimension. */

                if ( match( '[') ) {


                        /* Dimensioned array. */

                        if ( hexexp( &N) ) {


                                if ( !resolved )

                                        error( "unknown array dimension");


                                if ( match( ']') )

                                        dflag = DIMENSIONED;

                                else

                                        syntax( "[N] array dimension");

                                }


                        /* Undimensioned array. */

                        else if ( match( ']') )

                                { N = 0; dflag = UNDIMENSIONED; }


                        else

                                syntax( "{[N],[]} array dimension");

                        }



                else
                        /* Just a single variable. */

                        { N = 1; dflag = SINGLE; }






                /* Check for any initializers and advance the pc. */

                if ( match( '=') ) {


                        /* Make sure an output record is open and aligned. */

                        CHECK_RECORD();



                        /* Let any initializer emits advance the pc. */

                        if ( !initializer( type, dflag, N) )

                                syntax( "legal initializer");



                        /**** KEEP ALIGNED TO 4 BYTE BOUNDARIES. ****/

                        while ( pc&0x3 )

                                DATA( BYTE, 0L);

                        }


                else {

                        /* Advance the pc without any emits. */

                        pc += N*datasize[ type];



                        /**** KEEP ALIGNED TO 4 BYTE BOUNDARIES. ****/

                        pc = (pc+3)&~3L;


                        }


                }




        /* Check for the statement terminator. */

        if ( match( ',') )

                goto next;


        else if ( match( ';') )

                return( 1);

        else

                syntax( "legal variable name , or ;");




        return( 0); /* failure */




}
int initializer( int type, int dflag, unsigned long N) {

        unsigned long value;



        /* Emit initialized data. */


        if ( dflag == SINGLE ) {


                /* Initialize a single variable. */

                if ( initial_value( type, &value) )

                        { DATA( type, value); return( 1); }

                }



        else if ( dflag == DIMENSIONED || dflag == UNDIMENSIONED ) {


                /* Initialize an array. */

                if ( match( '{') ) {


                    next:


                        if ( initial_value( type, &value) ) {

                                DATA( type, value);

                                if ( dflag == DIMENSIONED && N-- == 0 )

                                        error( "too many initializers !");

                                }



                        /* Get another initializer value ? */

                        if ( match( ',') )

                                goto next;




                        /* If dimensioned, zero off any remaining entries. */

                        if ( dflag == DIMENSIONED )

                                while ( N-- > 0 ) DATA( type, 0L);




                        /* Get the initializer terminator. */

                        if ( match( '}') )

                                return( 1);

                        else

                                syntax( "legal {,,,} initializer");

                        }


                else

                        syntax( "{,,,} type initializer");


                }




        return( 0); /* failure */



}
int initial_value( int type, unsigned long *value) {


        switch ( type ) {


                case DSPF:

                        if ( dspf( (float *)value) )

                                return( 1);

                        break;


                case IEEE:

                        if ( ieee( (float *)value) )

                                return( 1);

                        break;

                default:

                        if ( hexexp( value) )

                                return( 1);

                        break;
                }



        return( 0); /* failure */



}
int function( void) {

        SYMBOL *s;




        /* Parse for a legal function declaration, func name { ... } */

        if ( !(s = enter( FUNC_SYM, gsymbol)) )

                return( 0);




        /* Finish entering the function name as a global symbol. */

        s->value.l = pc; s->flags |= RESOLVED;





        /* Make sure an output record is open and aligned. */

        CHECK_RECORD();




        /* Get the function body. */

        if ( match( '{') ) {


                /* Purge the local symbol table. */

                purge( lsymbol);



                /* Get any local declarations or statements. */

                next:
                        if ( local() || statement() )

                                     goto next;


                /* Check for the function terminator. */

                if ( match( '}') )

                        return( 1);

                else

                        syntax( "legal statement or closing function }");

                }


        else

                syntax( "legal function name { declaration");




        return( 0); /* failure */



}
int local( void) {



        /* Parse for a named register or structure declaration. */


        if ( named_register() || structure() )

                return( 1);



        return( 0); /* failure */




}
int named_register( void) {

        SYMBOL *s; int rN;



        /* Parse for a named register declaration. */

        if ( !scan( "register") )

                return( 0); /* failure */





        /* Enter register names as local symbols. */

  next:

        if ( (s = enter( REGISTER_SYM, lsymbol)) != NULL ) {


                if ( match( '=') && reg( &rN) )

                        { s->value.l = rN; s->flags |= RESOLVED; }


                else

                        syntax( "regname = rN  1 <= N <= 22");

                }





        /* Check for the statement terminator. */

        if ( match( ',') )

                goto next;


        else if ( match( ';') )

                return( 1);

        else

                syntax( "legal regname = rN [,...]; statement");




        return( 0); /* failure */





}
int structure( void) {


        unsigned long addr; SYMBOL *s; FIELD *head; int type;




        /* Parse for a structure declaration. */

        if ( !scan( "struct") )

                return( 0);



        /* Get the base address for the structure. */

        if ( !hexexp( &addr) )

                syntax( "legal N for struct N { ...");




        /* Enter field names as local symbols. */

        if ( !match( '{') )

                syntax( "struct N { ... declaration");




        head = NULL;

          next_type:


                if ( datatype( &type) ) {

                  next:


                        if ( find_field( head) )

                                syntax( "non-multiply defined field name");


                        else if ( (head = enter_field( head)) != NULL ) {

                                head->value = addr;

                                    addr += datasize[type];

                                }

                        else
                                syntax( "legal structure field name");




                        if ( match( ',') )

                                goto next;


                        else if ( match( ';') )

                                goto next_type;

                        else

                                syntax( "legal structure field , or ;");

                        }



                else if ( !match( '}') )

                        syntax( "struct N { data type ... } ...");





        /* Get the structure name. */

        if ( (s = enter( STRUCTURE_SYM, lsymbol)) != NULL ) {

                s->value.p = head;

                if ( resolved )

                        s->flags |= RESOLVED;

                }

        else

                syntax( "legal non-multiply defined structure name");






        /* Check for the statement terminator. */

        if ( match( ';') )

                return( 1);

        else

                syntax( "legal struct N {...} name; declaration");




        return( 0); /* failure */




}
int statement( void) {



        /* Parse for a conditional or instruction statement. */


        if ( conditional() || instruction() )

                return( 1);



        /* Check for a compound statement. */

        else if ( match( '{') ) {


                next:

                    if ( statement() )

                        goto next;



                /* Get the closing } on the compound statement. */

                if ( match( '}') )

                        return( 1);

                else

                        syntax( "statement or closing }");

                }



        /* Allow the null statement. */
        
        else if ( match( ';') )

                return( 1);




        return( 0); /* failure */
        


}





/* NOTE:
 *
 * The following variable do_level, and function check_do_level are for
 * avoiding nested do loops and conditional constructs within do loops
 * below.
 *
 *
 */


int
      do_level = 0;



int check_do_level( void) {


        if ( do_level > 0 )

                error( "no conditionals or calls inside do loops");


        return( 1); /* success */


}






int conditional( void) {

        int k, rH, rM, rS, c; unsigned long N, M, inc;

            char SRFAR *fixup, SRFAR *else_fixup, SRFAR *top;




        /* Parse for an if, while, for, ... conditional construct. */


        if ( conditional_keyword( &k) ) switch ( k ) {



                /* If statement, if ( cond ) statement [ else statement ] */ 

                case IF:


                        check_do_level();



                        /* Get the conditional test. */

                        if ( match( '(') && ifcond( &fixup) && match( ')') ) {


                                /* Get the body of the if statement. */

                                if ( !statement() )

                                        syntax( "if ( cond )");




                                /* Check for a possible following else. */

                                if ( scan( "else") ) {


                                    else_fixup = output;


                                        ELSE_BEGIN( fixup);


                                        if ( !statement() )

                                                syntax( "else {...}");


                                        ELSE_END( else_fixup);

                                        }

                                else

                                        IF_END( fixup);


                                return( 1);


                                }


                        else

                                syntax( "if ( cond ) {...}");


                        break;





                /* While loop, while ( cond ) statement */

                case WHILE:


                        check_do_level();


                        /* Get the conditional test at the top of while. */

                        top = output;


                        if ( match( '(') && ifcond( &fixup) && match( ')') ) {


                                if ( statement() )

                                        WHILE_END( top, fixup);

                                else

                                        syntax( "while ( cond ) {...}");

                                }


                        else
                                syntax( "while ( cond )");


                        break;





                /* For loop, for ( init ; cond ; inc ) statement */

                case FOR:


                        check_do_level();


                        /* Get any initializing expressions. */

                        if ( !match( '(') )

                                syntax( "for ( ...");


                        init:
                                if ( cau() && match( ',') )

                                        goto init;



                        if ( !match( ';') )

                                syntax( "for ( init ; ...");




                        /* Get the conditional test at the top of the loop. */

                        top = output;

                            ifcond( &fixup);


                        if ( !match( ';') )

                                syntax( "for ( init ; cond ; ...");




                        /* Get an optional incrementing ca expression. */

                        if ( cau() )

                                rewind_output( &inc);

                        else
                                inc = 0L;


                        if ( !match( ')') )

                                syntax( "for ( init ; cond ; inc ) ...");





                        /* Get the body of the for statement. */

                        if ( statement() )

                                FOR_END( top, fixup, inc);

                        else

                                syntax( "for ( init ; cond ; inc ) {...}");


                        break;






                /* Loop while, loop statement while ( rM-- >= 0 ); */

                case LOOP:


                        check_do_level();


                        /* Get the loop while statement body. */

                        top = output;


                        if ( !statement() )

                                error( "illegal loop while");




                        /* Get the trailing while. */

                        if ( scan( "while (") && reg( &rM) && scan( "-- >= 0 ) ;") ) {

                                LOOP_END( top, rM);

                                LOP();

                                }

                        else

                                syntax( "loop {...} while (rM-- >= 0);");


                        break;





                /* Do loops, do {rS+1,N} statement */

                case DO:


                        /* Make sure not already in a do loop. */

                        if ( do_level > 0 )

                                error( "cannot nest do loops");



                        /* Set global do_level to indicate in a do loop. */

                        do_level = 1;



                        /* Save the start of the do loop. */

                        fixup = output;



                        /* Get the do initialization. */

                        if ( reg( &rS) ) {


                                if ( scan( "+ 1") )

                                        DO_REG_BEGIN( rS);

                                else

                                        syntax( "do rS+1 {...}");

                                }



                        else if ( hexval( &N) )

                                DO_IMM_BEGIN( N);

                        else

                                syntax( "do {rS+1,(N)}");




                        /* Get the body of the do statement. */

                        if ( statement() )

                                { DO_END( fixup); do_level = 0; }
                        
                        else
                                
                                syntax( "do {rS+1,(N)} {...}");


                        break;


                }



        else
                return( 0); /* failure */




        return( 1); /* success */



}
int ifcond( char SRFAR **fixup) {

        int c, rD, inc_flag, inc_func, rS; unsigned long N;




        /* if ( cond ) .... */

        if ( cond( &c) )

                { *fixup = output; IF_BEGIN( c); LOP(); }





        /* if ( rD[++,--] relop {rS,N} [,inc_exp] ) ... */

        else if ( reg( &rD) ) {



                /* Get a possible post increment or decrement, rD[++,--]. */

                inc_flag = inc_dec( &inc_func);




                /* Get the compare condition. */

                if ( relop( &c) ) {


                        /* Emit a rD-{rS,N} compare to set the flags. */

                        if ( reg( &rS) )

                                CAU_6B( rD, DYADIC, NONE, FUNC_07, rS);


                        else if ( hexexp( &N) )

                                CAU_6D( rD, FUNC_07, N);


                        else

                                syntax( "rD[++,--] relop {rS,N}");




                        /* Test for the compare condition. */

                        *fixup = output; IF_BEGIN( c);



                        }


                else

                        syntax( "rD[++,--] {>,<,>=,<=,!=,==} {rS,N}");





                /* Either emit a ca expression, post inc, or latent nop. */

                if ( match( ',') ) {


                        if ( inc_flag )

                                error( "no ++ or -- with , post expression");


                        if ( !cau() )

                                syntax( "cau expression");

                        }



                else if ( inc_flag )

                        CAU_6B( rD, DYADIC, ZERO, inc_func, ONE);

                                                /** AT&T encoding **/

                else

                        LOP();

                }



        else

                syntax( "if ( cond )");




        return( 1); /* success */





}
int instruction( void) {

        int k, rH, rD, rIO, rS, s, c, aN; MEM m;

            unsigned long M, N, cau_instruction;

                SYMBOL *label;





    /* Parse for an instruction and terminating semicolon. */



        /* Keyword instructions, nop, call, return, etc, ... */

        if ( instruction_keyword( &k) ) switch ( k ) {



                /* Nop instruction, nop */

                case NOP:

                        NOPS( 1);

                        break;



                /* Call instruction, call {rH,rH+N,M} */

                case CALL:


                        check_do_level();



                        if ( reg( &rH) ) {


                                if ( match( '+') ) {


                                        if ( hexval( &N) ) 

                                                CAU_4( rH, N, RETURN_REG);

                                        else

                                                syntax( "call rH+(N)");

                                        }

                                else

                                        CAU_4( rH, 0, RETURN_REG);

                                }



                        else if ( hexexp( &M) )

                                CAU_8C( M, RETURN_REG);


                        else

                                syntax( "call {rH,rH+N,M}");



                        R32_CALL_FIX( RETURN_REG);

                        break;




                /* Return instruction, return (rM) */

                case RETURN:

                        CAU_0( NONE, RETURN_REG, 0);

                        LOP();

                        break;




                /* Goto instruction, goto {rH,rH+N,M} */

                case GOTO:


                        if ( reg( &rH) || pcreg( &rH) ) {


                                if ( match( '+') ) {


                                        if ( hexexp( &N) )

                                                CAU_8A( rH, N);

                                        else

                                                syntax( "goto rH+N");
                                        }

                                else

                                        CAU_8A( rH, 0L);

                                }



                        else if ( hexexp( &M) )

                                CAU_8A( ZERO, M);


                        else

                                syntax( "goto {rH,rH+N,M}");


                        LOP();

                        break;




                /* Ireturn instruction, ireturn */

                case IRETURN:

                        CAU_0( NONE, PC_SHADOW, 0);

                        LOP();

                        break;



                /* Cau latent nops on, nops_on */

                case NOPS_ON:

                        nops = 1;

                        break;




                /* Cau latent nops off, nops_off */

                case NOPS_OFF:

                        nops = 0;

                        break;



                /* Custom opcode emit, emit( N1, N2, N3, ... ) */

                case EMIT:


                        if ( match( '(') ) {

                            next:

                                if ( hexexp( &N) )

                                        emit( N);


                                if ( match( ',') )

                                        goto next;

                                else if ( !match( ')') )

                                        syntax( "emit( N1, N2, ... )");

                                }

                        else

                                syntax( "emit( ...");


                        break;





                /* Conditional cau exp, iff ( cacond ) cau_reg_exp ; */

                case IFF:


                        if ( match( '(') && cacond( &c) && match( ')') ) {


                                if ( cau() ) {


                                        rewind_output( &cau_instruction);

                                            IFF_END( cau_instruction, c);

                                        }


                                else

                                        syntax( "iff ( cacond ) cau_reg_exp");

                                }

                        else

                                syntax( "iff ( cacond )");

                        break;


                }






        /* Control register store, {ioc,dauc} = N */

        else if ( control( &rIO) ) {


                if ( match( '=') && hexexp( &N) )

                        CAU_5A( rIO, ZERO, N);

                else

                        syntax( "{ioc,dauc} = N");

                }





        /* Ioreg store, ioreg = {rS,*rP++rIr,aN}[.size] */

        else if ( ioreg1( &rIO) ) {


                if ( !match( '=') )

                        syntax( "ioreg = ...");



                if ( rIO == OBUF && dau( &Z_OBUF) )

                        /* obuf = aN = dau expression */ ;


                else if ( rIO == PDR && dau( &Z_PDR) )

                        /* pdr = aN = dau expression */ ;


                else if ( reg( &rS) )

                        { size( &s); CAU_7C( TO_IO, s, rIO, rS); }


                else if ( match( '*') && mem( &m) )

                        { size( &s); CAU_7D( FROM_MEM, s, rIO, &m); }



                else

                        syntax( "ioreg = {rS,*rP++rIr,aN}[.size]");

                }






        /* Memory store, {*rP++rIr,*N} = ... */

        else if ( match( '*') ) {



                /* Register indirect, *rP++rIr = {ioreg,rN,aN} ... */

                if ( mem( &m) ) {


                        if ( !match( '=') )

                                syntax( "*rP++rIr = {ioreg,rN,aN} ...");



                        if ( ioreg2( &rIO) )

                                { size( &s); CAU_7D( TO_MEM, s, rIO, &m); }


                        else if ( reg( &rH) )

                                { size( &s); CAU_7B( TO_MEM, s, rH, &m); }


                        else if ( dau( &m) )

                                /* *Z = aN = dau expression */ ;


                        else

                                syntax( "*rP++rIr = {ioreg,rN,aN} ...");

                        }



                /* Memory direct, *N = rH[.size] */

                else if ( hexval( &N) ) {


                        if ( match( '=') && reg( &rH) )

                                { size( &s); CAU_7A( TO_MEM, s, rH, N ); }


                        else
                                syntax( "*(N) = rH[.size]");

                        }


                else

                        syntax( "{*rP++rIr,*N} = ...");

                }





        /* CAU rD ... integer expression */

        else if ( cau() )

                ;




        /* DAU aN = ... floating point arithmetic expression, with no Z */

        else if ( dau( &NO_WRITE) )

                ;





        /* Goto label, label: */

        else if ( (label = enter( GOTO_SYM, gsymbol)) != NULL ) {


                if ( match( ':') ) {

                        label->value.l = pc; label->flags |= RESOLVED;

                            return( 1);

                        }


                else
                        syntax( "legal statement or label:");

                }



        else

                return( 0); /* failure */






        /* Check for the statement terminator. */

        if ( match( ';') )

                return( 1);

        else

                syntax( "terminating ;");





        return( 0); /* failure */





}
int cau( void) {

        int rD, rH, rS, rIO, func, s; MEM m; unsigned long N;




        /* Parse for cau expression, rD {=,op=,++,--,...} ... */


        if ( !reg( &rD) )

                return( 0); /* failure */




        /* Assignment, rD = ... cau type expression */

        if ( match( '=') ) {



                /* Ioreg fetch, rD = ioreg[.size] */

                if ( ioreg2( &rIO) )

                        { size( &s); CAU_7C( FROM_IO, s, rIO, rD); }




                /* Memory fetch, rD = {*rP++rIr,*N}[.size] */

                else if ( match( '*') ) {


                        if ( mem( &m) )

                                { size( &s); CAU_7B( TO_REG, s, rD, &m); }


                        else if ( hexval( &N) )

                                { size( &s); CAU_7A( TO_REG, s, rD, N); }


                        else

                                syntax( "rD = {*rP++rIr,*(N)}[.size]");

                        }



                /* Register negate, rD = -rS */

                else if ( match( '-') ) {


                        if ( reg( &rS) )

                                CAU_6B( rD, DYADIC, NONE, FUNC_05, rS);

                        else

                                syntax( "rD = -rS");

                        }




                /* PC counter operation, rD = {pc,pc+N} */

                else if ( pcreg( &rH) ) {


                        if ( match( '+') ) {


                                if ( hexexp( &N) )

                                        CAU_5B( rD, rH, N);

                                else

                                        syntax( "rD = pc+N");

                                }

                        else

                                CAU_5B( rD, rH, 0L); /* rD = pc */

                        }





                /* Triadic arithmetic and register load, rD = rS ... */

                else if ( reg( &rS) )

                        cau_arith( rD, rS);




                /* Right sum, rD = N {+,-} rH, and immediate load, rD = M */

                else if ( hexval( &N) ) {


                        if ( match( '+') ) {


                                if ( reg( &rH) ) /*** pcreg( &rH)? ***/

                                        CAU_5B( rD, rH, N);

                                else

                                        syntax( "rD = (N)+rH");

                                }


                        else if ( match( '-') ) {


                                if ( reg( &rS) && (rS == rD) )

                                        CAU_6D( rD, 0x2, N);

                                else

                                        syntax( "dyadic rD = (N)-rD");

                                }



                        /* Immediate load, 24 bit, rD = M */

                        else

                                CAU_8B( rD, N);

                        }




                else

                        syntax( "rD = ... cau type expression");

                }





        /* Increment, decrement, rD {++,--} */

        else if ( inc_dec( &func) )

                CAU_6B( rD, DYADIC, ZERO, func, ONE); /** AT&T encoding **/






        /* Compare/and with no store, rD {-,&} {rS,N} */

        else if ( compare( &func) ) {


                if ( reg( &rS) )

                        CAU_6B( rD, DYADIC, NONE, func, rS);


                else if ( hexval( &N) )

                        CAU_6D( rD, func, N);


                else

                        syntax( "rD {-,&} {rS,(N)}");

                }





        /* Dyadic arithmetic, rD {+=,-=,&=,|=,^=,#=,&~=} {rS,N} */

        else if ( dyadic_arith( &func) ) {


                if ( reg( &rS) )

                        CAU_6B( rD, DYADIC, NONE, func, rS);



                else if ( hexexp( &N) ) {


                        if ( func == FUNC_00 /* + */ )

                                CAU_5B( rD, rD, N); /** AT&T encoding **/

                        else

                                CAU_6D( rD, func, N);

                        }

                else

                        syntax( "rD {+,-,&,|,^,#,&~}= {rS,N}");

                }





        /* Dyadic arithmetic shift, rD {*=,/=} N, N = 0x2,0x4, ... */

        else if ( dyadic_ashift( &func) ) {


                if ( hexexp( &N) )

                        ASHIFT( rD, func, rD, N);


                else

                        syntax( "rD {*=,/=} N, N = 0x2,0x4, ...");

                }





        /* Dyadic logical shift, rD {>>=,>>>=,<<<=} N */

        else if ( dyadic_lshift( &func) ) {


                if ( hexexp( &N) )

                        LSHIFT( rD, func, rD, N);

                else

                        syntax( "rD {>>=,>>>=,<<<=} N, N <= 0x17");

                }





        else

                syntax( "rD {=,op=,++,--,...} ... cau expression");




        return( 1); /* success */




}
int cau_arith( int rD, int rS) {


        int func, rT; unsigned long N;




        /* Parse for cau arithmetic, rD = rS ... */




        /* Arithmetic functions, rD = rS {+,-,&,|,^,#,&=} {rT,N} */

        if ( arith( &func) ) {


                if ( reg( &rT) ) {


                        if ( rS == rD )

                                CAU_6B( rD, DYADIC, NONE, func, rT);


                        else
                                CAU_6B( rD, TRIADIC, rS, func, rT);


                        }


                else if ( hexval( &N) ) {


                        if ( func == FUNC_00 /* + */ )

                                CAU_5B( rD, rS, N);



                        else if ( func == FUNC_04 && rS != rD /* - */ )

                                CAU_5B( rD, rS, -N);



                        else if ( rS == rD )

                                CAU_6D( rD, func, N);


                        else

                                syntax( "rD = rD op (N), dyadic only");

                        }


                else

                        syntax( "rD = rS {+,-,&,|,^,#,&=} {rT,(N)}");

                }




        /* Arithmetic shift, rD = rS {*,/} N */

        else if ( ashift( &func) ) {


                if ( hexexp( &N) )

                        ASHIFT( rD, func, rS, N);

                else

                        syntax( "rD = rS {*,/} N, N = 0x2,0x4, ...");

                }




        /* Logical shift, rD = rS {>>,>>>,<<<} N */

        else if ( lshift( &func) ) {


                if ( hexexp( &N) )

                        LSHIFT( rD, func, rS, N);

                else

                        syntax( "rD = rS {>>,>>>,<<<} N, N <= 0x17");

                }




        /* Register load, rD = rS */

        else

              CAU_6B( rD, TRIADIC, rS, FUNC_00, ZERO); /** AT&T encoding **/






        return( 1); /* success, no failure possible ... */




}
int dau( MEM *Z) {



    /**
     *
     * DAU formats including aM = 0.0, 1.0 cases:
     *
     *    Fmt1              [Z=] aN = [-] Y {+,-} aM*X
     *    Fmt1  aM = 0.0    [Z=] aN = [-] Y
     *    Fmt1  aM = 1.0    [Z=] aN = [-] Y {+,-} X
     *    Fmt2                   aN = [-] aM {+,-} (Z=Y)*X
     *    Fmt2  aM = 0.0         aN = [-] (Z=Y)*X
     *    Fmt3              [Z=] aN = [-] aM {+,-} Y*X
     *    Fmt3  aM = 0.0    [Z=] aN = [-] Y*X
     *    Fmt4                   aN = [-] (Z=Y) {+,-} X
     *
     *    Fmt5              [Z=] aN = [-] special( Y)
     *
     *
     **/



        int aN, f, s, func; MEM A, B, Ztap;



        /* Parse for a dau expression, [Z] = aN = [-] {A,(,special} ... */


        if ( !acc( &aN) )

                return( 0);


        if ( !match( '=') )

                syntax( "[Z] = aN = [-] ... dau expession");




        /* Allow an optional - sign for the accumulator input. */

        sign( &f);




        /* Full mac, [Z] = aN = [-] A ... */

        if ( memXY( &A) )

                dau_arith( Z, aN, f, &A);





        /* Tap update, [Z] = aN = [-] (Ztap=A) {+,-,*} B */

        else if ( match( '(') ) {


                if ( !memZ( &Ztap) )

                        syntax( "aN = [-] (Ztap ...");


                if ( Z != &NO_WRITE )

                        error( "Z field specified twice");


                if ( Ztap.rI == DIRECT )

                        error( "Z tap cannot be an accumulator");




                if ( !(match( '=') && memXY( &A) && match( ')')) )

                        syntax( "aN = [-] (Ztap=A) ...");




                if ( sign( &s) ) {


                        if ( memXY( &B) )

                                DAU_4( aN, f, &Ztap, &A, s, &B);

                        else
                                syntax( "aN = [-] (Ztap=A) {+,-} B");

                        }

                else if ( match( '*') ) {


                        if ( memXY( &B) )

                                DAU_2( aN, 0, aM_ZERO, f, &Ztap, &A, &B);

                        else
                                syntax( "aN = [-] (Ztap=A)*B");

                        }




                else
                        syntax( "aN = [-] (Ztap=A) {+,-,*} ...");

                }





        /* Special function, [Z] = aN = [-] special( A) */

        else if ( special( &func) ) {


                if ( !f && match( '(') && memXY( &A) && match( ')') )

                        DAU_5( Z, aN, func, &A);

                else

                        syntax( "[Z] = aN = special( A)");

                }





        else

                syntax( "[Z] = aN = [-] {A,(,special} ...");




        return( 1); /* success */




}
int dau_arith( MEM *Z, int aN, int f, MEM *A) {

        int s; MEM B, C, Ztap;



        /* Parse for full mac, [Z] = aN = [-] A {+,-,*} ... */

        if ( sign( &s) ) {


                if ( memXY( &B) ) {


                        /* Full mac, [Z] = aN = [-] A {+,-} B*C */

                        if ( match( '*') ) {


                            if ( !memXY( &C) )

                                    syntax( "[Z] = aN = [-] A {+,-} B*C");



                                if ( A->rP == DIRECT && A->rI <= 3 )

                                        DAU_3( Z, aN, f, A->rI, s, &C, &B);


                                else if ( B.rP == DIRECT && B.rI <= 3 )

                                        DAU_1( Z, aN, f, A, s, B.rI, &C);


                                else if ( C.rP == DIRECT && C.rI <= 3 )

                                        DAU_1( Z, aN, f, A, s, C.rI, &B);


                                else

                                        error( "too many memory references");

                                }



                        /* [Z] = aN = [-] A {+,-} B */

                        else

                                DAU_1( Z, aN, f, A, s, aM_ONE, &B);

                        }




                /* Tap update, [Z] = aN = [-] A {+,-} (Ztap=B)*C */

                else if ( match( '(') ) {


                        if ( A->rP != DIRECT )

                            error( "aN = aM + ( ... required in tap update");



                        if ( !memZ( &Ztap) )

                                syntax( "aN = [-] aM {+,-} (Ztap ...");


                        if ( Z != &NO_WRITE )

                                error( "Z field specified twice");


                        if ( Ztap.rI == DIRECT )

                                error( "Z tap cannot be an accumlator");




                        if ( !(match( '=') && memXY( &B) && match( ')')) )

                                syntax( "aN = [-] aM {+,-} (Ztap=B) ...");




                        if ( match( '*') && memXY( &C) && A->rI <= 3 )

                                  DAU_2( aN, f, A->rI, s, &Ztap, &B, &C);

                        else

                                syntax( "aN = [-] aM {+,-} (Ztap=B)*C");

                        }



                else

                        syntax( "[Z] = aN = [-] A {+,-} {B,(} ...");

                }




        /* [Z] = aN = [-] A*B */

        else if ( match( '*') ) {

                if ( memXY( &B) )

                        DAU_3( Z, aN, 0, aM_ZERO, f, &B, A);

                else

                        syntax( "[Z] = aN = [-] A*B");
                }





        /* [Z] = aN = [-] A */

        else

                DAU_1( Z, aN, f, A, s, aM_ZERO, &NO_XY);




        return( 1); /* success, no failure possible ... */


}
