/* FILE: count.s                Copyright (c), Symmetric Research, 1991
 *
 * This program counts, filling an array with 1, 2, 3,..., and is written
 * in the AT&T assembler.
 *
 *
 * NOTES:
 *
 * When using the AT&T assembler, you must be VERY care about specifying
 * the rDe extension.  A 16 bit operation such as r1 = 0x8000 will have
 * the high bit sign extended to 0xFF8000 in the full 24 bit register.
 * 
 * This can cause inadvertant programming errors when loading addresses
 * as pointers.  The correct instruction in AT&T mnemonics would be
 * r1e = 0x8000, which would then load the 24 bit register with 0x008000.
 *
 * Also, be aware that in the AT&T assembler, nops for the CAU latency
 * must be explicitly specified.  Generally, they are required after all
 * memory loads and stores, and control instructions.
 *
 * These two issues are in contrast to the DSPASM assembler.  There all
 * CAU instructions are assembled by default to the full 24 bit versions
 * with no e required, and nops are automatically emitted for CAU
 * instructions requiring them.
 *
 *
 */

.rsect  ".main"

/* main()
 *
 * Fill an array in memory with 1, 2, ..., N
 *
 * In C pseudo code:
 *
 *      p = array; i = 1;
 *
 *      while ( n-- )
 *              *p++ = i++;
 *
 */

main:
        r1e = array;                    /* r1 -> array                  */
        r2 = 1;                         /* r2 = i                       */

        r3 = *n;                        /* r3 = loop counter            */
        nop;
        r3 = r3 - 1;

        goto check; nop;                /* while ( rN-- >= 0 ) { }      */

loop:           *r1++ = r2e;
                r2 = r2 + 1;

check:  if ( r3-- >= 0 ) goto loop; nop;


wait:   goto wait; nop;                 /* wait for the PC              */



/* Program data. */

#define N       10

n:      int     N

        .align  4

array:  N*int24 0




/* MORE NOTES:
 *
 * The AT&T assembler does not enforce any alignment of data.  When data
 * must be aligned on a 2 or 4 byte boundary, you must be careful to do so
 * explicitly with the .align directive.
 *
 * Generally, 24 and 32 bit operands *MUST* be aligned on 4 byte boundaries
 * in order for the processor to work correctly.  Misalinged long integer
 * and floating point data will not be fetched or stored correctly by the
 * processor.
 *
 * Again, this in contrast with the DSPASM assembler, which simply forces
 * all data objects to be aligned on 4 byte boundaries.
 *
 *
 */
