/* FILE: inner.c                Copyright (c), Symmetric Research, 1992-2000
 *
 * This program computes a vector inner product on the DSP32C, all under
 * PC control.  Functions from the DSPTOOLS DLL library are used to
 * download, start and stop execution, etc.
 *
 * Run inner.exe from the DOS command line, and the results will be
 * printed out.  Because the program uses the DSPTOOLS DLL library, the
 * system must be able to find dsptools.dll at runtime.  Make sure the
 * path \sr\dsp32c\dll is on your DOS environment execution path.
 * 
 * The PC must know some of the DSP32C data addresses.  You could
 * hardwire these into the C side of the program.  However, a much
 * better way is to include a DSPASM symbol table in the C code.  This
 * communicates the DSP32C addresses to the C program symbolically in an
 * easy to maintain way.  Run DSPASM with the /sc option to generate the
 * inner.dsc include file.
 *
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "dsptools.h"   /* <- ALWAYS include this when using DSPTOOLS */





/* DSP32C RESIDENT DATA:
 *
 * The include file inner.dsc has the addresses of the data in the
 * DSPASM portion of the program.  In this program there is only a a
 * single variable communicated between the two, the result of the inner
 * product.  
 *
 * You could hardwire DSP32C addresses into your C code, but using an
 * included symbol table is much easier to maintain.
 *
 */

#include "inner.dsc"






/* MAIN:
 *
 * Initialize, download, and run inner.dso on the DSP32C.
 *
 *
 */

int main( void) {

        float result;
        

        /* Initialize and download the DSP32C. */

        if ( !DSP32C_Init() )

              { printf( "ERROR: unable to initalize the DSP32C\n"); exit( 1); }


        if ( !DSP32C_SrLoad( "inner.dso", NO_REPORT) )

              { printf( "ERROR: unable to download inner.dso\n"); exit( 1); }




        /* Execute. */
        
        DSP32C_Go();

            while ( !DSP32C_Pir() )

                   /* wait for the PIR signal to indicate completetion ... */ ;

        DSP32C_Halt();
        


        
        /* Read the DSP32C result back and print it out. */
        
        DSP32C_Get( DSP_result, &result, sizeof( result));

        printf( "Inner product (should be 20.0) = %3.5f\n", result);


        exit( 0 );
}






/* MORE NOTES:
 *
 * Once the PC has started the DSP32C executing with the Go() function,
 * it must wait until the DSP is done before getting the result.
 *
 * Here this has been done by polling the pir register on the DSP32C PIO
 * port.  When the DSP32C starts its calculation, it stuffs the pir
 * register with 0, and when it is done, with a 1.  The PC just waits
 * until it sees pir = 1 before getting the result.
 *
 * This use of the pir register assumes the interrupt header on the
 * card is not hooked up to to the ISA bus.  If you are using interrupt
 * communication then this program will not work correctly.
 *
 *
 *
 */
