/* FILE: dspio.h                Copyright (c), Symmetric Research, 1995-1999
 *
 * SR DSP32C DSPIO low level function library include file.  It should
 * ALWAYS be included in C programs calling DSPIO.
 *
 *
 */




/* FUNCTION AND VARIABLE TYPES:
 *
 * These defines control the function and variable types for the SR DSPIO
 * library.
 *
 * Whether a function is marked for static, DLL export, or DLL import use
 * is controlled by the VISIBILITY define.  With the MS VC 6.0 compiler
 * some acceptable values are:
 *
 *      #define VISIBILITY
 *      #define VISIBILITY      __declspec( dllexport)
 *      #define VISIBILITY      __declspec( dllimport) 
 *
 *
 * The calling convention is controlled by the CALLCONV define.  With the
 * MS VC 6.0 compiler some acceptable values are:
 *
 *      #define CALLCONV
 *      #define CALLCONV        __stdcall
 *
 *
 * A null VISIBILITY string indicates standard static usage, while a
 * null CALLCONV string indicates C calling convention.
 *
 * We assume most users will want to use the library as a DLL from a
 * variety of languages, and so have defined dllimport and stdcall as
 * the defaults below.
 *
 * Users needing to override these defaults should give suitable defines
 * from inside their source code or the compiler command line.
 *
 *
 *
 */

#define VOID                    void
#define INT                     int
#define UINT                    unsigned int
#define I16                     short
#define I32                     long
#define U16                     unsigned short
#define U32                     unsigned long
#define PCHAR                   char *


#ifdef SROS_LINUX
#define ULONG                   unsigned long
#define UCHAR                   unsigned char
#define _STATIC 1
#undef _DLLEXPORT
#undef _DLLIMPORT
#endif


#ifdef _STATIC
#define VISIBILITY
#define CALLCONV
#endif

#ifdef _DLLEXPORT
#define VISIBILITY              __declspec( dllexport)
#define CALLCONV                __stdcall
#endif

#ifdef _DLLIMPORT
#define VISIBILITY              __declspec( dllimport)
#define CALLCONV                __stdcall
#endif

#ifndef VISIBILITY
#define VISIBILITY              __declspec( dllimport)
#endif

#ifndef CALLCONV
#define CALLCONV                __stdcall
#endif


#define VARTYPE( type)          VISIBILITY type
#define FUNCTYPE( type)         VISIBILITY type CALLCONV




/* Define __far, __near, and __huge keywords only for the 16 bit compiler.
 *
 * The 16 bit Microsoft C\C++ version 1.52 compiler sets _MSC_VER = 800, 
 * while the 32 bit Visual C++ version 6.0 compiler sets _MSC_VER = 1200.
 *
 */

#ifdef SROS_LINUX

#define __far
#define __near
#define __huge

#else

#if (_MSC_VER <= 800)

#define __far     _far 
#define __near    _near
#define __huge    _huge

#else

#define __far
#define __near
#define __huge

#endif // MSC_VER

#endif // SROS_LINUX





/* RELEASE REV INFO:
 *
 * This define, variable, and function allow the user to determine 
 * library rev at either compile or runtime.
 *
 * The rev is an integer, where a number like 134 would be referred to
 * as rev "1.34".
 *
 *
 */

#define DSP32C_REV              231

extern VARTYPE( INT) DSP32C_Rev;

FUNCTYPE( INT) DSP32C_GetRev( void);






/* DSPIO.C FUNCTIONS:
 *
 *
 *
 */

extern VARTYPE( INT) __near DSP32C_BaseIo;

FUNCTYPE(  INT) DSP32C_GetBaseIo( void);
FUNCTYPE( VOID) DSP32C_SetBaseIo( unsigned NewBaseIo);

FUNCTYPE(  INT) DSP32C_Init( void);

FUNCTYPE(  INT) DSP32C_Pcr( void);
FUNCTYPE(  INT) DSP32C_Pir( void);
FUNCTYPE(  INT) DSP32C_Esr( void);
FUNCTYPE(  INT) DSP32C_Emr( void);

FUNCTYPE( VOID) DSP32C_SetPir( unsigned value);
FUNCTYPE( VOID) DSP32C_SetEni( unsigned value);

FUNCTYPE( VOID) DSP32C_Go( void);
FUNCTYPE( VOID) DSP32C_Halt( void);

FUNCTYPE( VOID) DSP32C_Get( unsigned long src, void __huge *dest, unsigned long nbytes);
FUNCTYPE( VOID) DSP32C_Put( void __huge *src, unsigned long dest, unsigned long nbytes);
FUNCTYPE(  INT) DSP32C_SrHexLoad( unsigned long __far *input);

FUNCTYPE( VOID) DSP32C_PutLong( unsigned long value, unsigned long addr);







/* DSPIO.C ADDITIONAL DSPMUL MULTIPLE CPU BOARD FUNCTIONS:
 *
 * These items are only used by the DSPMUL multiple cpu board.
 *
 *
 */

extern VARTYPE( INT) __near DSP32C_BaseSel;

FUNCTYPE(  INT) DSP32C_GetBaseSel( void);
FUNCTYPE( VOID) DSP32C_SetBaseSel( unsigned NewBaseSel);
FUNCTYPE( VOID) DSP32C_SelectCpu( unsigned n);




/* DEVICE DRIVER FUNCTIONS:
 *
 * DSP32C_Open is automatically called by DSP32C_Init.
 *
 */

FUNCTYPE( INT) DSP32C_Open( void );
FUNCTYPE( INT) DSP32C_Close( void );

