/* FILE: config.dsp             Copyright (c), Symmetric Research, 1996
 *
 * This program scans DSPA64 multiplexer channels, filtering acquired
 * values and continuously saving them in a circular buffer.  Its is
 * essentially the same as iconfig.dsp with filtering added.
 *
 * The channels to be scanned and the order they are scanned in is
 * specified by a linked configuration list.  This linked configuration
 * list is a very flexible way to specify arbitrary scanning orders.
 *
 * For filtering, the same simple two term filter as in filter.dsp is
 * used.  The current value of the filter for each channel is saved in
 * the configuration list for use on the next iteration for that channel.
 *
 * Finally, note that this program is the core of the scope.exe program.
 *
 * To run this program download and execute config.dso by hand in the
 * DSPMON monitor, examining the VALUES array before and after running.
 *
 *
 *
 */




/* ASSEMBLY CONSTANTS:
 *
 *
 *
 */

constant

        /* Program locations. */

        RESET           = 0x000000,
        DATA            = 0x001000;





<RESET>

main {
        register

                waits           = r1,

                config          = r2,
                channel         = r3,
                value           = r4,
                scale           = r5,
                coeff           = r6;




        /* Program the DSP32C for 0 wait states. */

        waits = 0; pcw = waits;


        /* Initialize the serial port. */

        ioc = ILEN_16 | IN_MSB | OLEN_08 | OUT_MSB;




        /* Read 16 bit integer values from the DSPA64 channels:
         *
         *
         *
         *
         *
         *
         */

        config = C00;
         value = VALUES;

         scale = SCALE;
         coeff = COEFF;

  poll:

        if ( ibf ) { 

                                
                /* Set the mux channel. */

                channel = *config++;
                   obuf = channel.b;



                /* Get, filter, and save the new data value. */

                a1 = float( ibuf);

                        nop;
                        nop;
                        nop;

                a1 = *scale * a1;

                        nop;

                  a0 = a1 + *coeff * *config;

                        nop;


               *config++ = a2 = a0;
                *value++ = a2 = int( a0);

                        nop;




                /* Check for the end of the data array. */

                if ( value >= END_OF_VALUES )

                        value = VALUES;  /* reset back to beginning */




                /* Move on to the next configuration. */

                config = *config;


                }


        goto poll;


}







/* FILTER COEFFICIENTS AND IN MEMORY CONSTANTS:
 *
 *
 *
 *
 */

dspf    
        SCALE   = 0.20000000,           /* <- should be (1-COEFF) */
        COEFF   = 0.80000000,

        ZERO    = 0.00000000;





/* CONFIGURATION LIST:
 *
 * Each configuration list entry CXX[], has the next channel number to
 * use, the current filter value, and a pointer to the next
 * configuration entry to jump to.
 * 
 * The list is like that in iconfig.dsp, with the addition of the filter
 * value.
 * 
 * As in iconfig.dsp, the list below scans the first four channels,
 * where C00 in the C03[] should be changed to C04 to scan the entire
 * list.
 *
 *
 *
 */

long

        C00[] = { 0x00,  0x00000000,    C01 },
        C01[] = { 0x01,  0x00000000,    C02 },
        C02[] = { 0x02,  0x00000000,    C03 },
        C03[] = { 0x03,  0x00000000,    C00 },

        C04[] = { 0x04,  0x00000000,    C05 },
        C05[] = { 0x05,  0x00000000,    C06 },
        C06[] = { 0x06,  0x00000000,    C07 },
        C07[] = { 0x07,  0x00000000,    C08 },
        C08[] = { 0x08,  0x00000000,    C09 },
        C09[] = { 0x09,  0x00000000,    C0A },
        C0A[] = { 0x0A,  0x00000000,    C0B },
        C0B[] = { 0x0B,  0x00000000,    C0C },
        C0C[] = { 0x0C,  0x00000000,    C0D },
        C0D[] = { 0x0D,  0x00000000,    C0E },
        C0E[] = { 0x0E,  0x00000000,    C0F },
        C0F[] = { 0x0F,  0x00000000,    C10 },
        C10[] = { 0x10,  0x00000000,    C11 },
        C11[] = { 0x11,  0x00000000,    C12 },
        C12[] = { 0x12,  0x00000000,    C13 },
        C13[] = { 0x13,  0x00000000,    C14 },
        C14[] = { 0x14,  0x00000000,    C15 },
        C15[] = { 0x15,  0x00000000,    C16 },
        C16[] = { 0x16,  0x00000000,    C17 },
        C17[] = { 0x17,  0x00000000,    C18 },
        C18[] = { 0x18,  0x00000000,    C19 },
        C19[] = { 0x19,  0x00000000,    C1A },
        C1A[] = { 0x1A,  0x00000000,    C1B },
        C1B[] = { 0x1B,  0x00000000,    C1C },
        C1C[] = { 0x1C,  0x00000000,    C1D },
        C1D[] = { 0x1D,  0x00000000,    C1E },
        C1E[] = { 0x1E,  0x00000000,    C1F },
        C1F[] = { 0x1F,  0x00000000,    C20 },
        C20[] = { 0x20,  0x00000000,    C21 },
        C21[] = { 0x21,  0x00000000,    C22 },
        C22[] = { 0x22,  0x00000000,    C23 },
        C23[] = { 0x23,  0x00000000,    C24 },
        C24[] = { 0x24,  0x00000000,    C25 },
        C25[] = { 0x25,  0x00000000,    C26 },
        C26[] = { 0x26,  0x00000000,    C27 },
        C27[] = { 0x27,  0x00000000,    C28 },
        C28[] = { 0x28,  0x00000000,    C29 },
        C29[] = { 0x29,  0x00000000,    C2A },
        C2A[] = { 0x2A,  0x00000000,    C2B },
        C2B[] = { 0x2B,  0x00000000,    C2C },
        C2C[] = { 0x2C,  0x00000000,    C2D },
        C2D[] = { 0x2D,  0x00000000,    C2E },
        C2E[] = { 0x2E,  0x00000000,    C2F },
        C2F[] = { 0x2F,  0x00000000,    C30 },
        C30[] = { 0x30,  0x00000000,    C31 },
        C31[] = { 0x31,  0x00000000,    C32 },
        C32[] = { 0x32,  0x00000000,    C33 },
        C33[] = { 0x33,  0x00000000,    C34 },
        C34[] = { 0x34,  0x00000000,    C35 },
        C35[] = { 0x35,  0x00000000,    C36 },
        C36[] = { 0x36,  0x00000000,    C37 },
        C37[] = { 0x37,  0x00000000,    C38 },
        C38[] = { 0x38,  0x00000000,    C39 },
        C39[] = { 0x39,  0x00000000,    C3A },
        C3A[] = { 0x3A,  0x00000000,    C3B },
        C3B[] = { 0x3B,  0x00000000,    C3C },
        C3C[] = { 0x3C,  0x00000000,    C3D },
        C3D[] = { 0x3D,  0x00000000,    C3E },
        C3E[] = { 0x3E,  0x00000000,    C3F },
        C3F[] = { 0x3F,  0x00000000,    C00 };








/* STORAGE AREA FOR INCOMING DATA VALUES FROM THE DSPA64:
 *
 * Incoming values from all the scanned channels are saved in multiplexed
 * fashion in a single circular buffer.
 * 
 * Note that the dimension of VALUES should be a multiple of the number
 * of channels being scanned so that the multiplexed order is preserved
 * as the buffer is continuously circled.
 *
 *
 *
 */

constant NVALUES = 0x800;   /* SHOULD BE A MULTIPLE OF NCHANNELS! */

<DATA>

word

    VALUES[ NVALUES], END_OF_VALUES;













/* SERIAL PORT IOC CONTROL REGISTER BIT ASSIGMENTS:
 *
 * These constants define the bit assigments for the serial port IOC
 * configuration control register.  They provide convenient mnemonic names
 * for the bits.
 *
 *
 */

constant

        ILEN_00         = 0x0 << 0x06,          /* input length         */
        ILEN_08         = 0x1 << 0x06, 
        ILEN_16         = 0x2 << 0x06,
        ILEN_32         = 0x3 << 0x06,

        OLEN_00         = 0x0 << 0x0A,          /* output length        */
        OLEN_08         = 0x1 << 0x0A,
        OLEN_16         = 0x2 << 0x0A,
        OLEN_32         = 0x3 << 0x0A,

        IN_LSB          = 0x0 << 0x10,          /* input lst first      */
        IN_MSB          = 0x1 << 0x10,          /* input msb first      */

        OUT_LSB         = 0x0 << 0x11,          /* output lsb first     */
        OUT_MSB         = 0x1 << 0x11;          /* output msb first     */


