/* FILE: iconfig.dsp            Copyright (c), Symmetric Research, 1996
 *
 * This program scans the DSPA64 multiplexer channels, saving acquired
 * 16 bit integer values continuously in a circular buffer.
 * 
 * Rather than numerically computing the next channel as in switch.dsp
 * the channel seqeuence is taken from a configuration list.  This
 * provides complete flexibility, specifying the channel scanning order
 * without numerical scanning formulas.
 * 
 * The configuration list is comprised of entries specifying the next
 * channel to use and a pointer to the next configuration entry jump to.
 * Progressing on to the next configuration entry is a simple matter,
 * requiring only one instruction inside the polling loop.
 * 
 * No filtering or processing of the incoming data is done in this
 * program.  Incoming integer values are simply saved in multiplexed
 * fashion directly to memory.  See the next program, config.dsp for
 * adding filtering onto this basic model of scanning.
 * 
 * Finally, note the two sample delay that occurs when acquiring data.
 * The sample currently being saved into memory has the configuration
 * channel specified two samples earlier.
 * 
 *
 * To run this program, download and execute iconfig.dso by hand in the
 * DSPMON monitor.  Display the acquired data by hand with the dw
 * command while the program is running.
 *
 *
 *
 */




/* ASSEMBLY CONSTANTS:
 *
 *
 *
 */

constant

        /* Program locations. */

        RESET           = 0x000000,
        DATA            = 0x001000;




<RESET>

main {

        register

                waits           = r1,

                config          = r2,
                channel         = r3,
                value           = r4;





        /* Program the DSP32C for 0 wait states. */

        waits = 0; pcw = waits;


        /* Initialize the serial port. */

        ioc = ILEN_16 | IN_MSB | OLEN_08 | OUT_MSB;





        /* Read 16 bit integer values from the DSPA64 channels:
         *
         * This code is essentially the same as sample.dsp, only with
         * the channel configuration scanning added.
         *
         *
         */

        config = C00;
         value = VALUES;


  poll:

        if ( ibf ) { 

                                
                /* Set the mux channel. */

                channel = *config++;
                   obuf = channel.b;


                /* Save the data value. */

                *value++ = ibuf.w;



                /* Check for the end of the data array. */

                if ( value >= END_OF_VALUES )

                        value = VALUES;  /* reset back to beginning */



                /* Move on to the next configuration. */

                config = *config;

                }


        goto poll;


}






/* IN MEMORY CONSTANTS:
 *
 *
 *
 */

dspf    
        ZERO    = 0.00000000;






/* CONFIGURATION LIST:
 *
 * Each configuration list entry, CXX[], has the next channel number to
 * use and a pointer to the next configuration entry to jump to on the
 * next sample.
 *
 * You can specify any channel order either by changing the channel
 * number in each entry, or changing the next config pointers around.
 *
 * Later programs like config.dsp will include more fields in each CXX[]
 * entry.  Items like current filter status and parameters for each
 * configuration channel are typical additions.
 *
 * As listed below, the first four channels are scanned, where the C00
 * pointer in the line
 *
 *      C03[] = { 0x03,   C00 },
 *
 * jumps back to the beginning of the list.  Modify the C00 to C04 to
 * proceed on through the entire list.
 *
 *
 */

long

        C00[] = { 0x00,   C01 },
        C01[] = { 0x01,   C02 },
        C02[] = { 0x02,   C03 },
        C03[] = { 0x03,   C00 },

        C04[] = { 0x04,   C05 },
        C05[] = { 0x05,   C06 },
        C06[] = { 0x06,   C07 },
        C07[] = { 0x07,   C08 },
        C08[] = { 0x08,   C09 },
        C09[] = { 0x09,   C0A },
        C0A[] = { 0x0A,   C0B },
        C0B[] = { 0x0B,   C0C },
        C0C[] = { 0x0C,   C0D },
        C0D[] = { 0x0D,   C0E },
        C0E[] = { 0x0E,   C0F },
        C0F[] = { 0x0F,   C10 },
        C10[] = { 0x10,   C11 },
        C11[] = { 0x11,   C12 },
        C12[] = { 0x12,   C13 },
        C13[] = { 0x13,   C14 },
        C14[] = { 0x14,   C15 },
        C15[] = { 0x15,   C16 },
        C16[] = { 0x16,   C17 },
        C17[] = { 0x17,   C18 },
        C18[] = { 0x18,   C19 },
        C19[] = { 0x19,   C1A },
        C1A[] = { 0x1A,   C1B },
        C1B[] = { 0x1B,   C1C },
        C1C[] = { 0x1C,   C1D },
        C1D[] = { 0x1D,   C1E },
        C1E[] = { 0x1E,   C1F },
        C1F[] = { 0x1F,   C20 },
        C20[] = { 0x20,   C21 },
        C21[] = { 0x21,   C22 },
        C22[] = { 0x22,   C23 },
        C23[] = { 0x23,   C24 },
        C24[] = { 0x24,   C25 },
        C25[] = { 0x25,   C26 },
        C26[] = { 0x26,   C27 },
        C27[] = { 0x27,   C28 },
        C28[] = { 0x28,   C29 },
        C29[] = { 0x29,   C2A },
        C2A[] = { 0x2A,   C2B },
        C2B[] = { 0x2B,   C2C },
        C2C[] = { 0x2C,   C2D },
        C2D[] = { 0x2D,   C2E },
        C2E[] = { 0x2E,   C2F },
        C2F[] = { 0x2F,   C30 },
        C30[] = { 0x30,   C31 },
        C31[] = { 0x31,   C32 },
        C32[] = { 0x32,   C33 },
        C33[] = { 0x33,   C34 },
        C34[] = { 0x34,   C35 },
        C35[] = { 0x35,   C36 },
        C36[] = { 0x36,   C37 },
        C37[] = { 0x37,   C38 },
        C38[] = { 0x38,   C39 },
        C39[] = { 0x39,   C3A },
        C3A[] = { 0x3A,   C3B },
        C3B[] = { 0x3B,   C3C },
        C3C[] = { 0x3C,   C3D },
        C3D[] = { 0x3D,   C3E },
        C3E[] = { 0x3E,   C3F },
        C3F[] = { 0x3F,   C00 };








/* STORAGE AREA FOR INCOMING DATA VALUES FROM THE DSPA64:
 *
 * Incoming values from all the scanned channels are saved in multiplexed
 * fashion in a single circular buffer.
 * 
 * Note that the dimension of VALUES should be a multiple of the number
 * of channels being scanned so that the multiplexed order is preserved
 * as the buffer is continuously circled.
 *
 *
 *
 */

constant NVALUES = 0x800;   /* SHOULD BE A MULTIPLE OF NCHANNELS! */

<DATA>

word

    VALUES[ NVALUES], END_OF_VALUES;













/* SERIAL PORT IOC CONTROL REGISTER BIT ASSIGMENTS:
 *
 * These constants define the bit assigments for the serial port IOC
 * configuration control register.  They provide convenient mnemonic names
 * for the bits.
 *
 *
 */

constant

        ILEN_00         = 0x0 << 0x06,          /* input length         */
        ILEN_08         = 0x1 << 0x06, 
        ILEN_16         = 0x2 << 0x06,
        ILEN_32         = 0x3 << 0x06,

        OLEN_00         = 0x0 << 0x0A,          /* output length        */
        OLEN_08         = 0x1 << 0x0A,
        OLEN_16         = 0x2 << 0x0A,
        OLEN_32         = 0x3 << 0x0A,

        IN_LSB          = 0x0 << 0x10,          /* input lst first      */
        IN_MSB          = 0x1 << 0x10,          /* input msb first      */

        OUT_LSB         = 0x0 << 0x11,          /* output lsb first     */
        OUT_MSB         = 0x1 << 0x11;          /* output msb first     */


