/* FILE: sample18.dsp           Copyright (c), Symmetric Research, 1996
 *
 * This program is identical to sample.dsp, except that it reads a full
 * 18 bit integer value from the DSPA64.
 *
 * To achieve the 18 bit input, the program differs from the 16 bit program
 * sample.dsp in three places:
 *
 *      * initialization of the serial port, ioc = ...
 *      * reading ibuf, ie *value++ = ibuf.l
 *      * declaring the array VALUES to be longs
 *
 *
 * Note that the 18 bit values are aligned in the MSB's of the 32 bit
 * long integer memory location.
 *
 * Most of the SR DSPA64 application software is written to run in 16
 * bit integer mode.  This is the best comprimise between actual accuracy
 * of the DSP101, and ease of programming.
 *
 *
 * To run this program, download and execute iconfig.dso by hand in the
 * DSPMON monitor.  Display the acquired data by hand while the program
 * is executing.
 *
 *
 *
 */



/* ASSEMBLY CONSTANTS:
 *
 * These assembly time constants specify the program memory locations, and
 * channel selection.
 *
 *
 */

constant

        /* Program memory locations. */

        RESET           = 0x000000,
        DATA            = 0x000800,


        /* Channel selection. */

        CHANNEL         = 0x00;         /* 0x00 <= CHANNEL <= 0x3F */






<RESET>

main {
        register

                waits           = r1,

                channel         = r2,
                value           = r3;




        /* Program the DSP32C for 0 wait states. */

        waits = 0; pcw = waits;



        /* Initialize the serial port. */

        ioc = ILEN_32 | IN_MSB | OLEN_08 | OUT_MSB;




        /* Set the multiplexer channel:
         *
         * Note that constants cannot be loaded directly into obuf.  The
         * DSP32C instruction set only allows obuf to be loaded from a CAU
         * rN register memory.
         *
         *
         */

        channel = CHANNEL;
           obuf = channel.b;





        /* Read 16 bit integer values from the DSPA64:
         *
         * This loop polls the DSP32C serial input buffer ibuf, saving the
         * most significant 18 bits of acquired DSPA64 values to memory.
         *
         * The incoming data is stored in a circular buffer, where the input
         * pointer is reset to the beginning of the values array whenever the
         * end of the array is reached.
         *
         * No processing is done on incoming values.  They are simply saved
         * to memory.  Typical additional tasks might be converting integer
         * values to floating point, computing a digital filter, etc.  See
         * some of the other programs in this directory for examples.
         *
         *
         *
         */

        value = VALUES;

  poll:

        if ( ibf ) {


                /* Save the data value. */

                *value++ = ibuf.l;



                /* Check for the end of the array. */

                if ( value >= END_OF_VALUES )

                        value = VALUES;  /* reset back to beginning */

                }


        goto poll;

}








/* STORGE AREA FOR INCOMING DATA VALUES FROM THE DSPA64:
 *
 * The VALUES array is where incoming data is stored.  END_OF_VALUES marks
 * the end of the array.
 *
 * You may increase the size of the VALUES array until available memory
 * on your DSP card is exhausted.
 *
 *
 */

constant NVALUES = 0x100;

<DATA>

long
        VALUES[NVALUES] = { 0 }, END_OF_VALUES;











/* SERIAL PORT IOC CONTROL REGISTER BIT ASSIGMENTS:
 *
 * These constants define the bit assigments for the serial port IOC
 * configuration control register.  They provide convenient mnemonic names
 * for the bits.
 *
 *
 */

constant

        ILEN_00         = 0x0 << 0x06,          /* input length         */
        ILEN_08         = 0x1 << 0x06, 
        ILEN_16         = 0x2 << 0x06,
        ILEN_32         = 0x3 << 0x06,

        OLEN_00         = 0x0 << 0x0A,          /* output length        */
        OLEN_08         = 0x1 << 0x0A,
        OLEN_16         = 0x2 << 0x0A,
        OLEN_32         = 0x3 << 0x0A,

        IN_LSB          = 0x0 << 0x10,          /* input lst first      */
        IN_MSB          = 0x1 << 0x10,          /* input msb first      */

        OUT_LSB         = 0x0 << 0x11,          /* output lsb first     */
        OUT_MSB         = 0x1 << 0x11;          /* output msb first     */


