/* FILE: switch.dsp             Copyright (c), Symmetric Research, 1996
 *
 * This program collects 16 bit integer data from the DSPA64 board
 * while channel switching the multiplexer.  The acquired values are
 * saved continuously in a circular buffer in multiplexed fashion.
 *
 * The program is the simplest example of channel switching, where no
 * filtering or numerical processing is done on incoming data.
 *
 *
 * To run this program, download and execute switch.dso by hand in the
 * DSPMON monitor.  Display the acquired data by hand with the dw
 * command.
 *
 *
 *
 */



/* ASSEMBLY CONSTANTS:
 *
 *
 *
 */

constant

        /* Program memory locations. */


        RESET           = 0x000000,
        DATA            = 0x001000,


        /* Initial channel selection. */

        CHANNEL         = 0x00;         /* 0x00 <= CHANNEL <= 0x3F */




main {

        register

                waits           = r1,

                channel         = r2,
                value           = r3;





        /* Program the DSP32C for 0 wait states. */

        waits = 0; pcw = waits;


        /* Initialize the serial port. */

        ioc = ILEN_16 | IN_MSB | OLEN_08 | OUT_MSB;




        /* Read 16 bit integer data from succsive DSPA64 channels:
         *
         * The code has been written to switch between channel 0 and 1,
         * however you can easily change it to scan all 64 channels if you
         * desired.  Make sure the buffer length is a multiple of the number
         * of channels you are scanning.
         *
         * Note that there is a two sample delay from when the new channel
         * is specified to when that value is actually acquired.  This is
         * because the new channel is being sent WHILE the previous channel
         * is being saved and the current channel is being converted.
         *
         * See the manual for more discussion about the timing, and later
         * example programs for more sophisticated channel scanning.
         *
         *
         */

        channel = CHANNEL;
          value = VALUES;


  poll:

        if ( ibf ) { 

                                
                /* Set the next mux channel. */

                obuf = channel.b;


                /* Save the acquired value. */

                *value++ = ibuf.w;


                /* Move on to the next channel. */

                channel += 1; channel &= 0x1;


                /* Check for the end of the array. */

                if ( value >= END_OF_VALUES )

                        value = VALUES;  /* reset back to beginning */

                }


        goto poll;


}






/* STORGE AREA FOR INCOMING DATA VALUES FROM THE DSPA64:
 *
 * Incoming values from all the scanned channels are saved in multiplexed
 * fashion in a single circular buffer.
 * 
 * Note that the dimension of VALUES should be a multiple of the number
 * of channels being scanned so that the multiplexed order is preserved
 * as the buffer is continuously circled.
 *
 *
 *
 */

constant NVALUES = 0x800;   /* SHOULD BE A MULTIPLE OF NCHANNELS! */

<DATA>

word
        VALUES[NVALUES] = { 0 }, END_OF_VALUES;










/* SERIAL PORT IOC CONTROL REGISTER BIT ASSIGMENTS:
 *
 * These constants define the bit assigments for the serial port IOC
 * configuration control register.  They provide convenient mnemonic names
 * for the bits.
 *
 *
 */

constant

        ILEN_00         = 0x0 << 0x06,          /* input length         */
        ILEN_08         = 0x1 << 0x06, 
        ILEN_16         = 0x2 << 0x06,
        ILEN_32         = 0x3 << 0x06,

        OLEN_00         = 0x0 << 0x0A,          /* output length        */
        OLEN_08         = 0x1 << 0x0A,
        OLEN_16         = 0x2 << 0x0A,
        OLEN_32         = 0x3 << 0x0A,

        IN_LSB          = 0x0 << 0x10,          /* input lst first      */
        IN_MSB          = 0x1 << 0x10,          /* input msb first      */

        OUT_LSB         = 0x0 << 0x11,          /* output lsb first     */
        OUT_MSB         = 0x1 << 0x11;          /* output msb first     */

