// FILE: scope.h                Copyright (c), Symmetric Research, 2003
//
// Header file for generic portion of the Scope program includes
//
//      CScopeApp       main app    controls data, files, and registry
//      CScopeWindow    main window controls menus and graphics
//      CScopeDialog    base dialog provides validation + etc functions
//      CTriggerDlg     dialog for trigger options
//      COutputDlg      dialog for output options 
//      CDisplayDlg     dialog for display options 
//      CContentsDlg    dialog for help contents
//      CTopicDlg       dialog for help topics
//      CSrParser       parser support function
//



//***********************************************************************


// WINDOWS CONSTANTS

#include <afxwin.h>
#include <afxext.h>     // needed for status bar


// RESOURCE CONSTANTS

#include "scope.rch"



// APPLICATION CONSTANTS


#define WM_EDITLOSTFOCUS        (WM_USER+100)

#define UNCHECKED               0
#define CHECKED                 1
#define INDETERMINATE           2

#define MASK_CHECK              0X03L
#define MASK_HIGHLIGHT          0X04L
#define MASK_FOCUS              0X08L

#define IDT_TIMER               1

#define MSPERSEC                1000
#define TICKINBUFF              4

#define HMARGIN                 10
#define VMARGIN                 5

#define CS_MONO                 0
#define CS_GREEN                1
#define CS_BLUE                 2
#define CS_BROWN                3

#define FILE_NONE               0
#define FILE_ONE                1
#define FILE_SEQ                2
#define FILE_TIME               3

#define TRIG_NONE               0
#define TRIG_THRESHOLD          1
#define TRIG_DIGIO              2

#define MAXTITLE                12
#define MAXSTR                  256


#define DLG_POS_INIT            -999


// This is defined to be the maximum for all A/D's that
// will work with scope.  Currently two are supported:
//
//    A/D        Max Channels
//   DSPA64         64
//   PAR24B          8
//   PAR4CH          4
//

#define MAXHDRCHANNEL           64




struct channel_data {
        long  channel;
        char  title[MAXTITLE];
        };


struct HDR_DATA {
        char   Id[32];
        long   Channels;
        long   RecordPts;
        long   WordSize;
        long   DataCoding;
        double SampleRate;
        double Gain;
        double FilterCoeff;
        double FilterScale;
        struct channel_data ChannelList[MAXHDRCHANNEL];
        };

struct INI_DATA {
        
        CString IniFile;
        CString IniTitle;

        BOOL    DoAutoRun;
        BOOL    DoInteractive;
        BOOL    DoSysEnable;
        BOOL    DoStatusBar;
        BOOL    DoPlotTrace;
        BOOL    DoForbidMultiple;

        int     Nchannels;
        int     Nsamples;
        
        int     PltShiftBy;
        int     PltNdisplay;
        int     PltFirstSample;
        int     PltFirstTrace;
        int     PltColorScheme;
        BOOL    PltClipping;
        BOOL    PltAutoDC;

        int     AcqGainLog;
        float   AcqSPS;

        int     TrgType;
        int     TrgFilesToSave;
        int     TrgDigioLine;
        long    TrgThreshold;
        long    TrgMaxThreshold;
        
        int     OutFileMode;
        int     OutSeqStart;
        int     OutNblock;
        CString OutDir;
        
        long   *pScanActive;
        long   *pScanTrigger;
        int    *pScanOrder;
        char   *pScanTitle;

        };

// Needed for forward reference         
class CScopeWindow;             




//***********************************************************************


#define MAXTOKEN            256               /* token info */

#define TOKEN_EOF             0               /* token types */
#define TOKEN_SYMBOL          1
#define TOKEN_EQUALS          2
#define TOKEN_HEX             3
#define TOKEN_INTEGER         4
#define TOKEN_FLOAT           5
#define TOKEN_STRING          6
#define TOKEN_UNKNOWN         7

#define OFF  0
#define ON   1


class CSrParser : public CObject {

public:

        // Data
        
        FILE   *IniFp;
        char    IniName[ MAXSTR ];
        char    TOKEN[ MAXTOKEN ];
        char   *token;
        int     fline;
        int     ttype;
        int     c;


        // Methods
        
        CSrParser( void );

        int  OpenIni( const char *filename );
        int  CloseIni( void );
        int  ErrorIni( const char *msg );
        void AddToToken( int c );
        int  SkipSpace( int c );
        int  GetToken( void );
        int  Match( const char *s );
        int  KeywordEquals( const char *keyword );
        int  GetOnOff( int *parameter, const char *error );
        int  GetInt( int *parameter, const char *error );
        int  GetUint( unsigned int *parameter, const char *error );
        int  GetUlong( unsigned long *parameter, const char *error );
        int  GetHex( int *parameter, const char *error );
        int  GetFloat( double *parameter, const char *error );
        int  IsEOF( void );
};



//***********************************************************************


class CScopeApp : public CWinApp {
        
public:

        // Data

        int    DefaultColor;
        int    DefaultNblock;
        int    DefaultNdisplay;
        int    DefaultNsamples;
        int    DefaultThreshold;
        int    DataPtSize;
        int    MinChannel;
        int    MaxChannel;
        int    MaxSample;
        int    MinAmpShift;
        int    MaxAmpShift;
        int    MinGainLog;
        int    MaxGainLog;
        long   MaxTriggerThreshold;
        int    MaxTriggerDigioLine;
        float  MinSPS;
        float  MaxSPS;
        
        int    CurrBlock;
        int    CurrTriggerFile;
        int    NsamplesPlot;
        int    Iplotbuff;
        int    Nplotbuffers;

        long   Nbufferpts;
        long   NbufferptsPlot;
        long   Nbufferbytes;
        long   LastNbufferbytes;
        
        double *pTMPoffset;
        long   *pDCoffset;
        long   *pDemuxArray;

        BOOL   IsRunning;
        BOOL   IsAllocated;
        BOOL   ResetPaint;
        BOOL   ROnsamples;
        BOOL   ResetDC;
        BOOL   IsTriggered;

        int    ErrCode;
        HWND   hWndPrev;
        
        CString CurrDir;
        CString AppTitle;
        CString AppRev;
        CString AppClassName;
        CString ErrString;
        
        char szTemp[MAXSTR];                            

        struct HDR_DATA Hdr;
        struct INI_DATA Ini;
        struct INI_DATA IniSave;

        class CSrParser parse;


        
        // Methods

        //{{AFX_VIRTUAL(CScopeApp)
        virtual BOOL InitInstance();
        virtual int  ExitInstance();
        //}}AFX_VIRTUAL


        // Virtual A/D functions
        
        virtual int AtodMeasure( int *ival );
        virtual int AtodInit();
        virtual int AtodStart();
        virtual int AtodCheckReady();
        virtual int AtodCheckOverflow();
        virtual int AtodGetData();
        virtual int AtodDemuxData();
        virtual int AtodStop();

        
        // Virtual Buffer, File and Ini functions
                        
        virtual CScopeWindow* NewScopeWindow();
        virtual int InitAppData();
        virtual int IniVerifyData( void );
        virtual int IniSetDefaults( void );
        virtual int IniSaveCurrent( void );
        virtual int IniRestoreCurrent( void );
        virtual int IniIsChanged( void );
        virtual int IniClearChannels( void );
        virtual int IniAddChannel( int channel, char *title );
        virtual int IniReadKeyword( void );
        virtual int IniWriteKeyword( FILE *OutFp );
        virtual int ComputeActiveChannels( void );
        virtual int AllocateDataArray( void );
        virtual int TriggerCondition( void );
        virtual int TriggerThresholdSetRange( int chan, long *thmin, long *thmax );
        virtual int TriggerThresholdCompare( int pt, long thmin, long thmax );
        virtual int TriggerDigioCompare( int line, int activestate );
        virtual int FillHdr(  void );
        virtual int WriteBytes( CFile *OutFile );



        // Common File and Ini functions

        int  MultipleInstanceCondition( void );
        int  ComputeBufferSizes( void );
        int  AllocateBuffer( void );
        int  AllocateScanArrays( void );                        
        int  FreeScanArrays( void );
        int  WriteHdr(  void );
        int  WriteData( void );
        int  IniInputFromFile( void );
        int  IniOutputToFile( void );
        int  ShowHelp( int titleID, int textID, CWnd *caller );
        int  LoadHelpTitles( CListBox *listbox );
        int  AddResourceString( CListBox *listbox, int titleID, int textID, int pos = -1 );
        void ShowError();
        void ExitError();
        
                        
        //{{AFX_MSG(CScopeApp)
        //}}AFX_MSG


        DECLARE_MESSAGE_MAP()
};






//***********************************************************************

class CScopeWindow : public CFrameWnd {

        
public:

        // Data

        DWORD ColorBack;
        DWORD ColorAxis;
        DWORD ColorTitle;
        DWORD ColorTrace;
        
        CPen* PenBack;
        CPen* PenAxis;
        CPen* PenTitle;
        CPen* PenTrace;

        LOGFONT font;
        CFont*  CurrFont;

        int minPoint;
        int midPoint;
        int maxPoint;
        int cxChar;
        int cyChar;
        int MaxTitleLength;

        int DisplayPosTop;
        int DisplayPosLeft;
        int OutputPosTop;
        int OutputPosLeft;
        int ContentsPosTop;
        int ContentsPosLeft;
        int TopicPosTop;
        int TopicPosLeft;
        int TopicPosBottom;
        int TopicPosRight;
        int HardwarePosTop;
        int HardwarePosLeft;
        int ChannelPosTop;
        int ChannelPosLeft;
        int AcquirePosTop;
        int AcquirePosLeft;
        int TriggerPosTop;
        int TriggerPosLeft;

        int TraceStart;
        int TraceEnd;
        int TraceWidth;
        int TraceHeight;

        int *pYlast;
        int *pYaxis;
        int *pYclipmin;
        int *pYclipmax;

        long LastPtsPlot;

        POINT *pPtArray;
        POINT *pOldArray;

        
protected:
        CStatusBar m_wndStatusBar;
        CScrollBar m_wndVScroll;

        int  controlkey;
        BOOL IsDragging;
        BOOL PaintDeferred;


                
        // Methods
public:

        //{{AFX_VIRTUAL(CScopeWindow)
        virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
        virtual void PostNcDestroy();
        //}}AFX_VIRTUAL


        CScopeWindow();
        virtual ~CScopeWindow();

        void DrawBackground( CDC *pDC );
        void DrawTitles(     CDC *pDC );
        void DrawAxes(       CDC *pDC );
        void DrawTraces(     CDC *pDC );
        void FillTraces( int npoints );
        void UpdatePlotTrace();
        void UpdateStatusBar();
        void UpdateTitleTraces();
        void UpdateColorScheme();
        void SetColors();
        void CreatePens();
        void DeletePens();
        void CreateFont();
        void DeleteFont();
        void DeletePlotArrays();
        void DeleteVerticalBuffers();
        void UpdateTextSize( CDC *pDC );
        void SetClipLimits( int height, int traceHalf );
        void SetWindowTitle();
        void SetScrollBar( int nBar );
        void GetSize( int *height, int *width );
        int  AllocatePlotBuffers( );
        int  ChangeFontSize( int newsize );
        int  ComputeMaxTitleLength( CDC *pDC );
        int  ComputePlotPositions( );
        void InitDialogPositions( );
        void GetDialogPosition(  RECT *r, int nID );
        void SaveDialogPosition( RECT *r, int nID );


        

        //{{AFX_MSG(CScopeWindow)
        virtual int  OnCreate(LPCREATESTRUCT lpCreateStruct);
        afx_msg void OnSize(UINT nType, int cx, int cy);
        afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
        afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
        afx_msg void OnPaint();
        afx_msg void OnDestroy();
        afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
        afx_msg void OnKeyUp(  UINT nChar, UINT nRepCnt, UINT nFlags);
        afx_msg void OnTimer(UINT nIDEvent);
        afx_msg void OnRunProgram();
        afx_msg void OnStopProgram();
        afx_msg void OnExit();
        afx_msg void OnIniRead();
        afx_msg void OnIniWrite();
        afx_msg void OnSetClip();
        afx_msg void OnSetDC();
        afx_msg void OnSetAutoDC();
        afx_msg void OnSetStatusBar();
        afx_msg void OnSetPlotTrace();
virtual afx_msg void OnHardwareDialog();
virtual afx_msg void OnChannelDialog();
virtual afx_msg void OnAcquireDialog();
        afx_msg void OnDisplayDialog();
        afx_msg void OnOutputDialog();
        afx_msg void OnTriggerDialog();
        afx_msg void OnContentsDialog();
        afx_msg void OnAboutDialog();
        afx_msg void OnDebug();
        afx_msg void OnUpdateMenuRunning(   CCmdUI* pCmdUI);
        afx_msg void OnUpdateMenuStopped(   CCmdUI* pCmdUI);
        afx_msg void OnUpdateClipCheck(     CCmdUI* pCmdUI);
        afx_msg void OnUpdateAutoDCCheck(   CCmdUI* pCmdUI);
        afx_msg void OnUpdatePlotTraceCheck(CCmdUI* pCmdUI);
        //}}AFX_MSG

        afx_msg LRESULT OnEnterSizeMove( WPARAM wParam, LPARAM lParam);
        afx_msg LRESULT OnExitSizeMove(  WPARAM wParam, LPARAM lParam);
        
        DECLARE_MESSAGE_MAP()
};





//***********************************************************************


class CScopeDialog : public CDialog {
        

public:                                       
        CScopeDialog(UINT nIdTemplate, CWnd* pParent = NULL);

        
        // Data

        int  DialogId;
        int  CancelId;
        BOOL DoValidate;
                
protected:

        int   CheckFocus( void );
        int   ValidateAsInt(   int nID, int minval, int maxval, int *theval, char *txt );
        int   ValidateAsHex(   int nID, int minval, int maxval, int *theval, char *txt );
        int   ValidateAsLong(   int nID, long minval, long maxval, long *theval, char *txt );
        int   ValidateAsFloat( int nID, float minval, float maxval, float *theval, char *txt );
        int   SetDlgItemHex(   int nID, int   val );
        int   SetDlgItemFloat( int nID, float val );
        float GetDlgItemFloat( int nID, BOOL *lpTrans = NULL);
        BOOL  SetTextReadOnly( int nID );
        void  RestorePosition();
        void  RememberPosition();


        // Generated message map functions
        
        //{{AFX_MSG(CScopeDialog)
        afx_msg void OnActivateApp( BOOL bActive, HTASK hTask );
        virtual LONG OnEditLostFocus( UINT nID, LONG xxx );
        virtual BOOL OnInitDialog( int OkButtonId );
        virtual void OnCancel();
        virtual void OnOK();
        //}}AFX_MSG
        
        DECLARE_MESSAGE_MAP()
};



//***********************************************************************

class CAboutDlg : public CDialog {

public:
        CAboutDlg(CWnd* pParent = NULL);   // standard constructor

        //{{AFX_DATA(CAboutDlg)
        enum { IDD = IDD_ABOUTBOX };
        //}}AFX_DATA


protected:

        // Generated message map functions
        
        //{{AFX_MSG(CAboutDlg)
        virtual BOOL OnInitDialog();
        virtual void OnOK();
        //}}AFX_MSG
        
        DECLARE_MESSAGE_MAP()
};




//***********************************************************************


class CTriggerDlg : public CScopeDialog {


public:
        CTriggerDlg(CWnd* pParent = NULL);   // standard constructor
        ~CTriggerDlg();                      // destructor

        
        // Dialog Data

        int  *pTempTrigger;
        
        
        //{{AFX_DATA(CTriggerDlg)
        enum { IDD = IDD_TRIGGER };
        CEdit    m_TXT_TriggerFilesToSave;
        CEdit    m_TXT_TriggerThreshold;
        CEdit    m_TXT_TriggerDigioLine;
        CButton  m_OPT_TriggerNone;
        CButton  m_OPT_TriggerThresh;
        CListBox m_LST_Trigger;
        BOOL     m_HasDigio;
        int      m_Ntrigger;
        int      m_CurrChan;
        int      m_TriggerType;
        int      m_TriggerDigioLine;
        int      m_TriggerFilesToSave;
        int      m_TriggerTimeToSave;
        long     m_TriggerThreshold;
        //}}AFX_DATA



        // ClassWizard generated virtual function overrides
        //{{AFX_VIRTUAL(CTriggerDlg)
        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
        //}}AFX_VIRTUAL


protected:

        void InitData( void );
        void FillTriggerList( int selectchannel );
        void UpdateChannelData( int icurr );
        void UpdateInfo( void );
        int  AddItem( int ikey, int itriger );
        int  FindPos( void );
        void SetTriggerTypeRadio( void );
        void ComputeTriggerTimeToSave( void );

        
        // Generated message map functions
        
        //{{AFX_MSG(CTriggerDlg)
        virtual BOOL OnInitDialog();
        virtual void OnCancel();
        virtual void OnOK();
        virtual LONG OnEditLostFocus( UINT nID, LONG xxx );
        afx_msg void OnHelp();
        afx_msg void OnBtnChannels();
        afx_msg void OnBtnBack();
        afx_msg void OnBtnNext();
        afx_msg void OnBtnAllOn();
        afx_msg void OnBtnAllOff();
        afx_msg void OnChkOnOff();
        afx_msg void OnChangeTxtCurr();
        afx_msg void OnKillfocusTxtCurr();
        afx_msg void OnKillfocusTxtTriggerFilesToSave();
        afx_msg void OnKillfocusTxtTriggerThreshold();
        afx_msg void OnKillfocusTxtTriggerDigioLine();
        afx_msg void OnSelchangeTriggerList();
        afx_msg void OnOptTriggerType( );
        //}}AFX_MSG
        DECLARE_MESSAGE_MAP()


};





//***********************************************************************


class COutputDlg : public CScopeDialog {
        

public:                                       
        COutputDlg(CWnd* pParent = NULL);                       // standard 
        COutputDlg(BOOL ROnsamples, CWnd* pParent = NULL);      // special

        // Dialog Data
        
        //{{AFX_DATA(COutputDlg)
        enum { IDD = IDD_OUTPUT };
        BOOL    m_ROnsamples;
        int     m_Nblock;
        int     m_Nsamples;
        int     m_FileOut;
        int     m_SeqStart;
        CString m_WorkDir;
        //}}AFX_DATA


        // Overrides
        
        // ClassWizard generated virtual function overrides
        //{{AFX_VIRTUAL(COutputDlg)
        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
        //}}AFX_VIRTUAL


                
protected:

        void InitData();
        void ComputeFileSize();


        // Generated message map functions
        
        //{{AFX_MSG(COutputDlg)
        virtual BOOL OnInitDialog();
        virtual void OnCancel();
        virtual void OnOK();
        virtual LONG OnEditLostFocus( UINT nID, LONG xxx );
        afx_msg void OnHelp();
        afx_msg void OnOptFileOut();
        afx_msg void OnKillfocusTxtNblock();
        afx_msg void OnKillfocusTxtNsample();
        afx_msg void OnKillfocusTxtSeqStart();
        afx_msg void OnKillfocusTxtWorkDir();
        //}}AFX_MSG
        
        DECLARE_MESSAGE_MAP()
};





//***********************************************************************


class CDisplayDlg : public CScopeDialog {


public:
        CDisplayDlg(CWnd* pParent = NULL);   // standard constructor

        
        // Dialog Data
        
        //{{AFX_DATA(CDisplayDlg)
        enum { IDD = IDD_DISPLAY };
        CEdit   m_TXT_ShiftBy;
        CEdit   m_TXT_Ndisplay;
        CEdit   m_TXT_FirstSample;
        CEdit   m_TXT_FirstTrace;
        CStatic m_LAB_PlotBits;
        CStatic m_LAB_Nscan;
        CStatic m_LAB_LastSample;
        CButton m_CHK_Clipping;
        CButton m_CHK_AutoDC;
        CButton m_OPT_ColorMono;
        CButton m_OPT_ColorGreen;
        CButton m_OPT_ColorBrown;
        CButton m_OPT_ColorBlue;
        int     m_ShiftBy;
        int     m_Ndisplay;
        int     m_FirstSample;
        int     m_FirstTrace;
        int     m_ColorScheme;
        int     m_TraceWidth;
        BOOL    m_Clipping;
        BOOL    m_AutoDC;
        //}}AFX_DATA



        // ClassWizard generated virtual function overrides
        //{{AFX_VIRTUAL(CDisplayDlg)
        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
        //}}AFX_VIRTUAL


protected:

        void SetPlotBitsLabel( void );
        void SetNscanLabel( void );
        void SetLastSampleLabel( void );
        void SetColorSchemeRadio( void );

        
        // Generated message map functions
        
        //{{AFX_MSG(CDisplayDlg)
        virtual BOOL OnInitDialog();
        virtual void OnCancel();
        virtual void OnOK();
        virtual LONG OnEditLostFocus( UINT nID, LONG xxx );
        afx_msg void OnHelp();
        afx_msg void OnKillfocusTxtShiftby();
        afx_msg void OnKillfocusTxtNdisplay();
        afx_msg void OnKillfocusTxtFirstsample();
        afx_msg void OnKillfocusTxtFirsttrace();
        afx_msg void OnChkClipping( );
        afx_msg void OnChkAutoDC( );
        afx_msg void OnOptColorScheme( );
        //}}AFX_MSG
        DECLARE_MESSAGE_MAP()


};





//***********************************************************************


class CContentsDlg : public CScopeDialog {

public:
        CContentsDlg(CWnd* pParent = NULL);   // standard constructor

        // Dialog Data
        
        //{{AFX_DATA(CContentsDlg)
        enum { IDD = IDD_HELPCONTENTS };
        CListBox        m_LST_topics;
        //}}AFX_DATA


        // ClassWizard generated virtual function overrides
        //{{AFX_VIRTUAL(CContentsDlg)
        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
        //}}AFX_VIRTUAL

        
protected:

        // Generated message map functions
        
        //{{AFX_MSG(CContentsDlg)
        afx_msg void OnBtnDisplay();
        afx_msg void OnHelp();
        virtual void OnCancel();
        virtual BOOL OnInitDialog();
        afx_msg void OnDblclkTopics();
        //}}AFX_MSG
        
        DECLARE_MESSAGE_MAP()
};






//***********************************************************************

class CTopicDlg : public CScopeDialog {

public:

        // Data
        
        CString htitle;
        CString htext;
        int     m_TitleID;
        int     m_TextID;

        //{{AFX_DATA(CTopicDlg)
        enum { IDD = IDD_HELPTOPIC };
        CEdit   m_TXT_Topic;
        //}}AFX_DATA

        

        // Methods
        
        CTopicDlg(CWnd* pParent = NULL);                          // standard
        CTopicDlg(int TitleID, int TextID, CWnd* pParent = NULL); // new


        // ClassWizard generated virtual function overrides
        
        //{{AFX_VIRTUAL(CTopicDlg)
        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
        //}}AFX_VIRTUAL


protected:

        // Generated message map functions
        
        //{{AFX_MSG(CTopicDlg)
        virtual BOOL OnInitDialog();
        afx_msg void OnSize(UINT nType, int cx, int cy);
        afx_msg void OnSetfocusTopic();
        //}}AFX_MSG
        
        DECLARE_MESSAGE_MAP()
};
